#!/bin/ksh

# Script to automate the tuning of the IBM Directory Server versions 6.0, 5.2 and 4.1
# for IBM Tivoli Access Manager for e-business.  It includes tuning IDS LDAP and
# DB2.

# This script only works on the non-secure (non-SSL) ports of the directory server
# and directory admin server

# Restrictions:
# - This script must be run under the context of the root system user.
# - It has only been tested on Unix platforms, primarily AIX, Solaris and Linux.
# - It does not require write authority to the current directory.

# Make sure "." is in the path.
export PATH=.:$PATH

# BEGIN customize section

# Setup up parameters (customize these)
# Prompts will be given for changing these default values.

#    The following variables is the default for backing up the database.
backup_dir_before=/db2_backup/before_tuning
backup_dir_after=/db2_backup/after_tuning

#    The following variables control spreading the database across multiple disk drives or
#    file systems.  This is also referred to as containerizing (my appologies to the language).
#    Only enable this if the database is to be spread accross multiple drives or file systems.
if [ 0 = 1 ];then
# Example 1
backup_dir_cont=/db2_backup/containerizing_temp
tablespace2containers="(path '/db2/disk1/ldapdb2_containers/tblspc2', path '/db2/disk2/ldapdb2_containers/tblspc2', path '/db2/disk3/ldapdb2_containers/tblspc2')"
tablespace3containers="(path '/db2/disk1/ldapdb2_containers/tblspc3', path '/db2/disk2/ldapdb2_containers/tblspc3', path '/db2/disk3/ldapdb2_containers/tblspc3')"
fi
if [ 0 = 1 ];then
# Example 2
backup_dir_cont=/db2_backup/containerizing_temp
tablespace2containers="(path '/disks/1/db2containers/tblspc2',  path '/disks/2/db2containers/tblspc2',
			path '/disks/3/db2containers/tblspc2',  path '/disks/4/db2containers/tblspc2',
			path '/disks/5/db2containers/tblspc2',  path '/disks/6/db2containers/tblspc2',
			path '/disks/7/db2containers/tblspc2',  path '/disks/8/db2containers/tblspc2',
			path '/disks/9/db2containers/tblspc2',  path '/disks/10/db2containers/tblspc2',
			path '/disks/11/db2containers/tblspc2', path '/disks/12/db2containers/tblspc2',
			path '/disks/13/db2containers/tblspc2', path '/disks/14/db2containers/tblspc2',
			path '/disks/15/db2containers/tblspc2'
			)"
tablespace3containers="(path '/disks/1/db2containers/tblspc3',  path '/disks/2/db2containers/tblspc3',
			path '/disks/3/db2containers/tblspc3',  path '/disks/4/db2containers/tblspc3',
			path '/disks/5/db2containers/tblspc3',  path '/disks/6/db2containers/tblspc3',
			path '/disks/7/db2containers/tblspc3',  path '/disks/8/db2containers/tblspc3',
			path '/disks/9/db2containers/tblspc3',  path '/disks/10/db2containers/tblspc3',
			path '/disks/11/db2containers/tblspc3', path '/disks/12/db2containers/tblspc3',
			path '/disks/13/db2containers/tblspc3', path '/disks/14/db2containers/tblspc3',
			path '/disks/15/db2containers/tblspc3'
			)"
fi
if [ 0 = 1 ];then
# Example 3
tablespace1containers="(path '/disk1/db2cont/tblspc1', path '/disk2/db2cont/tblspc1', path '/disk3/db2cont/tblspc1', path '/disk4/db2cont/tblspc1')"
tablespace2containers="(path '/disk1/db2cont/tblspc2', path '/disk2/db2cont/tblspc2', path '/disk3/db2cont/tblspc2', path '/disk4/db2cont/tblspc2')"
tablespace3containers="(path '/disk1/db2cont/tblspc3', path '/disk2/db2cont/tblspc3', path '/disk3/db2cont/tblspc3', path '/disk4/db2cont/tblspc3')"
fi
if [ 0 = 1 ];then
# Example 4
tablespace2containers="(path '/uservg/db2_cont1/tblspc2', path '/uservg/db2_cont2/tblspc2')"
tablespace3containers="(path '/uservg/db2_cont1/tblspc3', path '/uservg/db2_cont2/tblspc3')"
fi
if [ 0 = 1 ];then
# Example 5
tablespace0containers="(path '/hdisk2fs/idsldap1_containers/tblspc0')"
tablespace1containers="(path '/hdisk2fs/idsldap1_containers/tblspc1')"
tablespace2containers="(path '/hdisk2fs/idsldap1_containers/tblspc2')"
tablespace3containers="(path '/hdisk2fs/idsldap1_containers/tblspc3')"
fi
if [ 1 = 1 ];then
# Example 7
tablespace2containers="(path '/hdisk2fs/idsldap3_containers/tblspc2')"
tablespace3containers="(path '/hdisk2fs/idsldap3_containers/tblspc3')"
fi

#    The following variable controls detailed logging.  If enabled, the output
#    of most commands is put to the log file
if [ X$detailed_logging = X ];then
	detailed_logging=1
fi

# END customize section

# export key variables
export detailed_logging
export ldapvers
export ldapserv
export ldap_def_inst
export idsdirctl
export idsdiradm
export ldap_def_conf
export logfile
export logcmd
export ibmslapd_conf
export idsldap_inst
export ldap_adm_port
export ldap_port
export ldap_binddn
export ldap_pwd
export ldap_cred
export ldap_adm_cred
export ldapdbinst
export ldapdbuser
export ldapdbname
export tabschema
export ldapinstloc
export backup_dir
export db2vers
export db2fp 
export db2_ksh_running

if [ `uname` = "SunOS" ];then
	AWK=nawk
else
	AWK=awk
fi

OS=`uname -s`

# Get ldap version
which idsldapsearch 2>&1 >/dev/null
if [ $? = 0 ];then
	ldapvers=`idsldapsearch -e | grep "SDK Version:" | $AWK '{print $NF}'`
else
	ldapvers=`ldapsearch -e | grep "SDK Version:" | $AWK '{print $NF}'`
fi
ldapvers=$(( $ldapvers + 0 ))

if [ $ldapvers -ge 600 ];then
	ldapserv=idsslapd
	# default to the first instance
	ldap_def_inst=`idsilist 2>&1 | tail +2 | head -1`
	idsdirctl=idsdirctl
	idsdiradm=idsdiradm
elif [ $ldapvers -ge 500 ];then
	ldapserv=ibmslapd
	ldap_def_conf=/etc/ibmslapd.conf
	idsdirctl=ibmdirctl
	idsdiradm=ibmdiradm
else
	ldapserv=slapd
	ldap_def_conf=/etc/slapd32.conf
fi

if [ "X$logfile" = X ];then
	script_name_short=`print $0 | $AWK -F. '{if ($1==$NF){print $1}else
		{print substr($0,1,length($0)-length($NF)-1)}}'`
	logfile=$script_name_short".out"
	#rm -f $logfile
fi

if [ $detailed_logging = 1 ];then
	logcmd="tee -a $logfile"
else
	logfile=/dev/null
	logcmd="cat"
fi

if [ "X$db2_ksh_running" = X ];then
	db2_ksh_running=0
fi

askif(){
	q=$1
	recommend=$2

	doit=2
	while [ $doit = 2 ];do
		print -n $q" (y/n) [$recommend]? "
		read ans
		print $q" (y/n) [$recommend]? "$ans >> $logfile

		if [ X$ans = X ];then
			ans=$recommend
		fi

		if [ X$ans = Xy ];then
			doit=1
		fi
		if [ X$ans = Xn ];then
			doit=0
		fi
	done
}


# Create a background ksh process under the db2 instance owner user context for
# processing DB2 commands.  Sourcing in the db2profile does not work in all
# cases or all versions of DB2.

# The main case that does not work with sourcing in the db2profile is queries
# and updates on the DB2 system statistic tables.

# Note that kicking off another background process will interfere with the
# pipe for this one, so be careful.

start_db2_ksh(){

	if [ $db2_ksh_running = 0 ];then

		db2_ksh_running=1

		print "\tsu - $ldapdbuser -c ksh"
		su - $ldapdbuser -c ksh |&

		print "\t. ~$ldapdbinst/sqllib/db2profile"
		print -p ". ~$ldapdbinst/sqllib/db2profile"

		print "\tdb2 connect to $ldapdbname >/dev/null"
		print -p "db2 connect to $ldapdbname >/dev/null"

		print -p "print $0: done"

		read -p db2_ksh_out
		while [ "X$db2_ksh_out" != "X$0: done" ];do
			read -p db2_ksh_out
		done
	fi
}

# routine to send a command to the db2 ksh

write_db2_ksh(){
	print "\t$1"
	print -p "$1"
	print -p "print $0: done"
}

# routine to read the db2 ksh output

read_db2_ksh(){

	db2_ksh_verbose=0
	if [ "X$1" = "Xverbose" ];then
		db2_ksh_verbose=1
	fi

	read -p db2_ksh_out
	while [ "X$db2_ksh_out" != "X$0: done" ];do
		if [ $db2_ksh_verbose = 1 ];then
			print $db2_ksh_out
		fi
		db2_ksh_result=$db2_ksh_out
		read -p db2_ksh_out
	done
}

# stop the background db2 ksh process

stop_db2_ksh(){
	if [ $db2_ksh_running != 0 ];then
		write_db2_ksh "db2 terminate"
		read_db2_ksh verbose
		print -p exit
		sleep 1
		db2_ksh_running=0
	fi
}

# for IDS 5.2 and older
ask_ldap_conf_file(){

	# Removed (using cat<< EOF) the asking for a config file name.  Only supporting default.

	cat << EOF >/dev/null # removed
######## REMOVED ##############################

	while [ X$ibmslapd_conf = X ];do

		print "" | $logcmd
		print -n "Enter the ldap config file name (default $ldap_def_conf): "
		read ans
		print "Enter the ldap config file name (default $ldap_def_conf): "$ans >> $logfile

		if [ X$ans = X ];then
			ans=$ldap_def_conf
		fi
######## REMOVED ##############################
EOF
		ans=$ldap_def_conf

		if [ -f $ans ];then

			# Determine whether there is permission on the ldap config file
			# Looks weird, but this routes standard error to the output
			# and throws away standard out.
			if [ X"`wc -l $ans 2>&1 >/dev/null`" != X ];then
				print "" | $logcmd
				print "The ldap config file is not accessable or is empty" | $logcmd
				wc -l $ans 2>&1 >/dev/null | $logcmd
				print "This script should be run under the context of the system root user." | $logcmd
				print "" | $logcmd
				current_user=`id | $AWK -F"(" '{print $2}' | $AWK -F")" '{print $1}'`
				print "The currently running user is: "$current_user | $logcmd
				exit 1
			fi
			ibmslapd_conf=$ans
		else
			print "The $ans file does not exist.  The directory server may not be installed or configured." | $logcmd
			exit 1
		fi

		cat << EOF >/dev/null # removed
######## REMOVED ##############################
		else
			print "File \"$ans\" does not exist.  Try again" | $logcmd
		fi
	done
######## REMOVED ##############################
EOF
}

# for IDS 6.0 and above, but also checks and works with 5.2 and 4.1 to get the ldap conf file
ask_ldap_inst(){

	if [ $ldapvers -ge 600 ];then

		while [ X$idsldap_inst = X ];do

			# get the instance list
			ids_inst_list=`idsilist 2>&1 | tail +2`
			ids_inst_cnt=`idsilist 2>&1 | tail +2 | wc -l`

			if [ "X$ids_inst_list" = X ];then
				print "" | $logcmd
				print "There are no defined instances or the idsilist command failed" | $logcmd
				idsilist 2>&1 | $logcmd
				print "This script should be run under the context of the system root user." | $logcmd
				print "" | $logcmd
				current_user=`id | $AWK -F"(" '{print $2}' | $AWK -F")" '{print $1}'`
				print "The currently running user is: "$current_user | $logcmd
				exit 1
			fi

			# if there is only one instance configured, use it
			if [ $ids_inst_cnt = 1 ];then

				ans=$ldap_def_inst

			else
				# Otherwise, choose from the list

				print "" | $logcmd
				print "The following directory server instances are configured (using idsilist):" | $logcmd
				idsilist 2>&1 | tail +2 | $logcmd

				print "" | $logcmd
				print -n "Enter the name of the directory server instance to be used for this session (default $ldap_def_inst): "
				read ans
				print "Enter the name of the directory server instance to be used for this session (default $ldap_def_inst): "$ans >> $logfile

				if [ X$ans = X ];then
					ans=$ldap_def_inst
				fi
			fi

			match=`echo $ids_inst_list | $AWK -v ans=$ans '{print index($0,ans)}'`
			if [ $match != 0 ];then
				idsldap_inst=$ans

				# from the LDAP instance, determine the location of the LDAP config file
				print "Obtaining instance location" | $logcmd
				echo "\tidsilist -r 2>&1 | tail +2 | $AWK -F\"#\" -v inst=$idsldap_inst '{if (\$1==inst){print \$3}}'" | $logcmd
				idsilist -r 2>&1 | tail +2 | $AWK -F"#" -v inst=$idsldap_inst '{if ($1==inst){print "\t"$3}}' | $logcmd
				ldapinstloc=`idsilist -r 2>&1 | tail +2 | $AWK -F"#" -v inst=$idsldap_inst '{if ($1==inst){print $3}}'`

				# use the instance location to determine the location of the ldap config file
				ibmslapd_conf=$ldapinstloc/idsslapd-$idsldap_inst/etc/ibmslapd.conf

				# if the file does not exist, then ask for it
				if [ ! -f $ibmslapd_conf ];then
					ldap_def_conf=$ibmslapd_conf
					ask_ldap_conf_file
				fi
			else
				print "The specified directory server instance \"$ans\" does not exist.  Try again" | $logcmd
			fi
		done
	else
		ask_ldap_conf_file
	fi
}


ask_ldapport(){

while [ X$ldap_port = X ];do

	if [ $ldapvers -ge 600 ];then

		ask_ldap_inst

		print "Obtaining port number for instance" | $logcmd
		echo "\tidsilist -r 2>&1 | tail +2 | $AWK -F\"#\" -v inst=$idsldap_inst '{if (\$1==inst){print \$6}}'" | $logcmd
		idsilist -r 2>&1 | tail +2 | $AWK -F"#" -v inst=$idsldap_inst '{if ($1==inst){print "\t"$6}}' | $logcmd
		ldap_port=`idsilist -r 2>&1 | tail +2 | $AWK -F"#" -v inst=$idsldap_inst '{if ($1==inst){print $6}}'`

		# no longer need this, since not using idsdirctl
		if [ 0 = 1 ];then
			print "Obtaining admin daemon port number for instance" | $logcmd
			echo "\tidsilist -r 2>&1 | tail +2 | $AWK -F\"#\" -v inst=$idsldap_inst '{if (\$1==inst){print \$8}}'" | $logcmd
			idsilist -r 2>&1 | tail +2 | $AWK -F"#" -v inst=$idsldap_inst '{if ($1==inst){print "\t"$8}}' | $logcmd
			ldap_adm_port=`idsilist -r 2>&1 | tail +2 | $AWK -F"#" -v inst=$idsldap_inst '{if ($1==inst){print $8}}'`
		fi

	else # for IDS 5.2 and 4.1

		ask_ldap_conf_file

		print "Obtaining the port number" | $logcmd
		print "\tcat $ibmslapd_conf | $AWK '{" | $logcmd
		print "\t	if (\$1==\"dn:\"){dn=\$2}" | $logcmd
		print "\t	if (toupper(\$1)==toupper(\"ibm-slapdPort:\") &&" | $logcmd
		print "\t		toupper(dn)==toupper(\"cn=Configuration\")){print \$2}}'" | $logcmd
		cat $ibmslapd_conf | $AWK '{
			if ($1=="dn:"){dn=$2}
			if (toupper($1)==toupper("ibm-slapdPort:") &&
				toupper(dn)==toupper("cn=Configuration")){print "\t"$2}}' | $logcmd
		ldap_port=`cat $ibmslapd_conf | $AWK '{
			if ($1=="dn:"){dn=$2}
			if (toupper($1)==toupper("ibm-slapdPort:") &&
				toupper(dn)==toupper("cn=Configuration")){print $2}}'`

		if [ X$ldap_port = X ];then
			print "Error: Server ibm-slapdPort not found in $ibmslapd_conf file" | $logcmd
			unset ibmslapd_conf
		fi

		# no longer need this, since not using idsdirctl
		if [ 0 = 1 ];then
			if [ $ldapvers -ge 500 ];then
				print "Obtaining the admin daemon port number" | $logcmd
				print "\tcat $ibmslapd_conf | $AWK '{" | $logcmd
				print "\t	if (\$1==\"dn:\"){dn=\$2}" | $logcmd
				print "\t	if (toupper(\$1)==toupper(\"ibm-slapdPort:\") &&" | $logcmd
				print "\t		toupper(dn)==toupper(\"cn=Admin,\")){print \$2}}'" | $logcmd
				cat $ibmslapd_conf | $AWK '{
					if ($1=="dn:"){dn=$2}
					if (toupper($1)==toupper("ibm-slapdPort:") &&
						toupper(dn)==toupper("cn=Admin,")){print "\t"$2}}' | $logcmd
				ldap_adm_port=`cat $ibmslapd_conf | $AWK '{
					if ($1=="dn:"){dn=$2}
					if (toupper($1)==toupper("ibm-slapdPort:") &&
						toupper(dn)==toupper("cn=Admin,")){print $2}}'`

				if [ X$ldap_adm_port = X ];then
					print "Error: Admin ibm-slapdPort not found in $ibmslapd_conf file" | $logcmd
					unset ibmslapd_conf
				fi
			fi
		fi
	fi
done
}

ask_ldap_binddn(){

	while [ X$ldap_binddn = X ];do

		ask_ldap_inst

		print "Obtaining the LDAP root administrator dn" | $logcmd
		echo "\tcat $ibmslapd_conf | $AWK '{if (toupper(\$1)==toupper(\"ibm-slapdAdminDN:\")){print \$2}}'" | $logcmd
		cat $ibmslapd_conf | $AWK '{if (toupper($1)==toupper("ibm-slapdAdminDN:")){print "\t"$2}}' | $logcmd
		ldap_binddn=`cat $ibmslapd_conf | $AWK '{if (toupper($1)==toupper("ibm-slapdAdminDN:")){print $2}}'`

		if [ X$ldap_binddn = X ];then
			print "Error: ibm-slapdAdminDN not found in $ibmslapd_conf file" | $logcmd
			unset ibmslapd_conf
		fi
	done
}

ask_ldap_cred_500plus(){

	ask_ldapport
	ask_ldap_binddn

	while [ X"$ldap_cred" = X ];do

		print "" | $logcmd
		while [ X$ldap_pwd = X ];do

			print -n "Enter the ldap root administrator's password: "
			read ans
			print "Enter the ldap root administrator's password: "$ans >> $logfile

			if [ X$ans != X ];then
				ldap_pwd=$ans
			fi
		done

		# ensure that the admin server is running
		echo "$idsdiradm -I $idsldap_inst 2>/dev/null" | $logcmd
		$idsdiradm -I $idsldap_inst 2>/dev/null

		# verify the credential
		echo "$idsdirctl -p $ldap_adm_port -D $ldap_binddn -w $ldap_pwd status >/dev/null 2>&1" | $logcmd
		$idsdirctl -p $ldap_adm_port -D $ldap_binddn -w $ldap_pwd status >/dev/null 2>&1
		if [ $? = 0 ];then
			ldap_cred="-p $ldap_port -D $ldap_binddn -w $ldap_pwd"
			ldap_adm_cred="-p $ldap_adm_port -D $ldap_binddn -w $ldap_pwd"
		else
			unset ldap_pwd
		fi
	done
}

# for IDS 4.1
get_ldap_pid(){
	# Assumes the ldap configuration file and port have already been obtained
	# through calls to the following:
	#   ask_ldap_inst
	#   ask_ldapport

	ldap_pid=0
	ldap_pid1=`ps -e -o pid,args | $AWK -v pid=$ldapserv '{
		if (index($2,pid)){print $1}}'`
	ldap_pid2=`ps -e -o pid,args | $AWK -v pid=$ldapserv -v conf=$ibmslapd_conf '{
		if (index($2,pid) && $4 == conf){print $1}}'` 
	if [ X"$ldap_pid1" != X ];then
		ldapsearch -p $ldap_port -s base -b "" "objectclass=*" >/dev/null 2>&1
		if [ $? = 0 ];then
			if [ `print $ldap_pid1 | wc -w` = 1 ];then
				ldap_pid=$ldap_pid1
			else if [ X"ldap_pid2" != X ];then
				ldap_pid=$ldap_pid2
			fi;fi
		fi
	fi
	if [ X$ldap_pid = X ];then
		ldap_pid=0
	fi
	print $ldap_pid
}

# Returns a zero status code if running
is_ldap_running(){

	ask_ldap_inst
	ask_ldapport

	if [ 0 = 1 ];then
		# Old way: Decided that using idsdirctl to start the directory server
		# is not as easy to use.  It requires the credential, which makes more
		# sense when starting in remote mode, but not locally.

		if [ $ldapvers -ge 500 ];then
			# Old way using idsdirctl

			# this call also makes sure the admin server is running
			ask_ldap_cred_500plus

			# determine whether the directory server is running
			$idsdirctl $ldap_adm_cred statusreturn >/dev/null 2>&1

			return $?
		fi
	fi

	# New way using ldapsearch only
	if [ $ldapvers -ge 600 ];then

		# Determine whether it is already up (handles a null base search)
		already_up=0
		ldapsearch -p $ldap_port -s base -b "" "objectclass=*" >/dev/null 2>&1

		return $?
	else
		ldap_pid=`get_ldap_pid`
		if [ $ldap_pid != 0 ];then
			# is running
			return 0
		else
			# is not running
			return 1
		fi
	fi
}


# Stop the ldap server
stop_ldap(){
	if [ 0 = 1 ];then
		# Old way: Decided that using idsdirctl to start the directory server
		# is not as easy to use.  It requires the credential, which makes more
		# sense when starting in remote mode, but not locally.

		if [ $ldapvers -ge 500 ];then
			# Old way using idsdirctl

			# if the server is running, stop it
			is_ldap_running
			if [ $? = 0 ];then
				echo $idsdirctl $ldap_adm_cred stop | $logcmd
				$idsdirctl $ldap_adm_cred stop 2>&1 | $logcmd
			fi

			# wait for the server to stop
			sleep 2
			is_ldap_running
			while [ $? = 0 ];do
				sleep 2
				is_ldap_running
			done
		fi
	fi

	# Using the -k option or kill

	# if the server is running, stop it
	is_ldap_running
	if [ $? = 0 ];then

		print $0: Stopping the directory server : `date` | $logcmd
		if [ $ldapvers -ge 600 ];then
			echo $ldapserv -I $idsldap_inst -k | $logcmd
			$ldapserv -I $idsldap_inst -k 2>&1 | $logcmd
		else
			echo kill -9 $ldap_pid | $logcmd
			kill -9 $ldap_pid | $logcmd
		fi

		# wait for the server to stop
		sleep 2
		is_ldap_running
		while [ $? = 0 ];do
			sleep 2
			is_ldap_running
		done
		print $0: The directory server is stopped : `date` | $logcmd
	fi
}

start_ldap(){
	# Start the directory server
	ask_ldap_inst
	ask_ldapport

	if [ 0 = 1 ];then
		# Old way: Decided that using idsdirctl to start the directory server
		# is not as easy to use.  It requires the credential, which makes more
		# sense when starting in remote mode, but not locally.

		if [ $ldapvers -ge 500 ];then
			# Old way using idsdirctl

			# this call also makes sure the admin server is running
			ask_ldap_cred_500plus

			# determine whether the directory server is running
			already_up=0

			# if running, make sure not in config mode 
			is_ldap_running
			if [ $? = 0 ];then

				# get the config mode status
				ans=`ldapsearch -p $ldap_port -s base -b "" "objectclass=*" ibm-slapdisconfigurationmode | grep ibm-slapdisconfigurationmode | $AWK -F= '{print $2}'`

				# if in config mode, stop then restart the server
				if [ X$ans = XTRUE ];then
					stop_ldap
				else
					already_up=1
				fi
			fi

			# if not already up
			if [ $already_up = 0 ];then
				# now start the server
				echo $idsdirctl $ldap_adm_cred start | $logcmd
				$idsdirctl $ldap_adm_cred start 2>&1 | $logcmd

				# wait for the server to start
				sleep 2
				is_ldap_running
				while [ $? != 0 ];do
					sleep 2
					is_ldap_running
				done

				# Make sure it really is up (handles a null base search)
				ldapsearch -p $ldap_port -s base -b "" "objectclass=*" >/dev/null 2>&1
				while [ $? != 0 ];do
					sleep 2
					ldapsearch -p $ldap_port -s base -b "" "objectclass=*" >/dev/null 2>&1
				done

				# Make sure it really really is up (not just in configuration mode)
				ans=`ldapsearch -p $ldap_port -s base -b "" "objectclass=*" ibm-slapdisconfigurationmode | grep ibm-slapdisconfigurationmode | $AWK -F= '{print $2}'`

				# if in config mode, mention it
				if [ X$ans = XTRUE ];then
					print "LDAP server failed to start in full function mode.  It is running in configuration mode only." | $logcmd
				fi
			fi
		fi
	fi # end of old way

	if [ $ldapvers -ge 500 ];then

		# Use idsslapd or ibmslapd to start the server

		# Determine whether the directory server is already running
		already_up=0

		# if running, make sure not in config mode 
		is_ldap_running
		if [ $? = 0 ];then

			# get the config mode status
			ans=`ldapsearch -p $ldap_port -s base -b "" "objectclass=*" ibm-slapdisconfigurationmode | grep ibm-slapdisconfigurationmode | $AWK -F= '{print $2}'`

			# if in config mode, stop it before starting it
			if [ X$ans = XTRUE ];then
				stop_ldap
			else
				already_up=1
			fi
		fi

		# if not already up
		if [ $already_up = 0 ];then

			# now start the server
			print $0: Starting the directory server : `date` | $logcmd
			if [ $ldapvers -ge 600 ];then
				echo $ldapserv -I $idsldap_inst | $logcmd
				$ldapserv -I $idsldap_inst
			else
				echo $ldapserv -f $ibmslapd_conf | $logcmd
				$ldapserv -f $ibmslapd_conf
			fi

			# wait for the server to start
			sleep 2
			is_ldap_running
			while [ $? != 0 ];do
				sleep 2
				is_ldap_running
			done

			# Make sure it really really is up (not just in configuration mode)
			ans=`ldapsearch -p $ldap_port -s base -b "" "objectclass=*" ibm-slapdisconfigurationmode | grep ibm-slapdisconfigurationmode | $AWK -F= '{print $2}'`

			# if in config mode, mention it
			if [ X$ans = XTRUE ];then
				print "LDAP server failed to start in full function mode.  It is running in configuration mode only." | $logcmd
			else
				print $0: The directory server is started : `date` | $logcmd
			fi
		fi
	else # for IDS 4.1

		# if not running, start it
		is_ldap_running
		if [ $? != 0 ];then
			print $0: Starting the directory server : `date` | $logcmd
			echo $ldapserv -f $ibmslapd_conf | $logcmd
			$ldapserv -f $ibmslapd_conf

			# wait for the server to start
			sleep 2
			is_ldap_running
			while [ $? != 0 ];do
				sleep 2
				is_ldap_running
			done
			print $0: The directory server is started : `date` | $logcmd
		fi
	fi
}

# side effect:  starts LDAP
ask_ldap_cred(){

	if [ 0 = 1 ];then
	# old way:  uses idsdirctl
		if [ $ldapvers -ge 500 ];then
			# ask_ldap_cred_500plus
			start_ldap # also gets the ldap credential
		fi
	fi

	start_ldap
	ask_ldapport
	ask_ldap_binddn

	while [ X"$ldap_cred" = X ];do

		print "" | $logcmd

		while [ X$ldap_pwd = X ];do

			print -n "Enter the ldap root administrator's password: "
			read ans
			print "Enter the ldap root administrator's password: "$ans >> $logfile

			if [ X$ans != X ];then
				ldap_pwd=$ans
			fi
		done

		echo "ldapsearch -p $ldap_port -D $ldap_binddn -w $ldap_pwd -s base -b \"\" \"objectclass=*\" ibmdirectoryversion" | $logcmd
		print "\t"`ldapsearch -p $ldap_port -D $ldap_binddn -w $ldap_pwd -s base -b "" "objectclass=*" ibmdirectoryversion 2>&1 | grep -i ibmdirectoryversion | $logcmd`
		ldapsearch -p $ldap_port -D $ldap_binddn -w $ldap_pwd -s base -b "" "objectclass=*" ibmdirectoryversion >/dev/null
		if [ $? = 0 ];then
			ldap_cred="-p $ldap_port -D $ldap_binddn -w $ldap_pwd"
		else
			unset ldap_pwd
		fi
	done
}

# start the LDAP server in config mode
start_ldap_config_mode(){

	ask_ldap_inst

	if [ $ldapvers -ge 600 ];then
		print "Starting the IDS LDAP server in configuration mode, if not already running" | $logcmd
		echo $ldapserv -I $idsldap_inst -a >/dev/null 2>&1 | $logcmd
		$ldapserv -I $idsldap_inst -a >/dev/null 2>&1
	elif [ $ldapvers -ge 500 ];then
		print "Starting the IDS LDAP server in configuration mode, if not already running" | $logcmd
		echo $ldapserv -f $ibmslapd_conf -a >/dev/null 2>&1 | $logcmd
		$ldapserv -f $ibmslapd_conf -a >/dev/null 2>&1
	else
		start_ldap
	fi
}

# Get the LDAP cred and put the server in config mode.  If config mode is
ask_ldap_cred_and_start_config_mode_if_supported(){
	if [ "X$ldap_cred" = X ];then
		# old way
		if [ 0 = 1 ];then
			if [ $ldapvers -ge 500 ];then
				ask_ldap_cred_500plus
				start_ldap_config_mode
			fi
		fi

		# new way starts the server to get credential
		ask_ldap_cred # also starts the ldap server
	else
		# if not running, start it in config mode
		is_ldap_running
		if [ $? != 0 ];then
			start_ldap_config_mode
		fi
	fi
}

# This was pulled out of ask_ldapdbinst, since if called before the directory server
# has been started the first time, the results will be null for the tabschema
get_tabschema(){

	# Get the db2 database schema.  It is not clear whether this is the same as the
	# ldapdbuser or not.  Some db2 database restores indicated there would be problems
	# with this assumption, if the restored schema was not the same as the configured
	# one.  It might just require a reconfigure, to do such a restore.  This would
	# probably not be a big problem.

	# Use db2 table list instead to get the schema
	print "Obtaining the database schema name" | $logcmd

	echo "\tdb2 connect to $ldapdbname >/dev/null" | $logcmd
	db2 connect to $ldapdbname >/dev/null

	if [ $? = 4 ];then
		# db2 has not been started

		# start it
		echo db2start | $logcmd
		db2start | $logcmd

		# and try again
		echo "\tdb2 connect to $ldapdbname >/dev/null" | $logcmd
		db2 connect to $ldapdbname >/dev/null

		if [ $? != 0 ];then
			print "Error:  Could not connect to the database" | $logcmd
			exit 1
		fi
	fi

	# old way:
	#echo "\tdb2 list tables for all | $AWK '{if (\$1==\"LDAP_ENTRY\"){print \$2;exit}}'" | $logcmd
	#db2 list tables for all | $AWK '{if ($1=="LDAP_ENTRY"){print ""$2;exit}}' >$0.tmp
	#tabschema=`cat $0.tmp`
	#rm $0.tmp

	# new way:
	echo "\tdb2 -x \"select tabschema from syscat.tables where tabname = 'LDAP_ENTRY'\""
	db2 -x "select tabschema from syscat.tables where tabname = 'LDAP_ENTRY'" |&
	wait $!
	read -p tabschema

	echo "\tdb2 terminate >/dev/null" | $logcmd
	db2 terminate >/dev/null

	print "\t$tabschema" | $logcmd
}

ask_ldapdbinst(){

	while [ X$ldapdbinst = X ];do

		ask_ldap_inst

		# Old way: start the server in configuration mode and get information.
		# Decided not to do this because it requires the ldap admin password
		# and some uses of this script are just for db2 tuning, which
		# does not require the LDAP server.
		# old way: if [ $ldapvers -ge 500 ];then
		if [ 0 = 1 ];then
			ask_ldap_cred_500plus

			# if the directory server is not running, start it in config mode
			is_ldap_running
			running_status=$?
			if [ $running_status != 0 ];then
				# start in config mode
				start_ldap_config_mode
			fi

			# get the database instance name
			ldapdbinst=`ldapsearch $ldap_cred -s base -b "cn=Directory, cn=RDBM Backends, cn=IBM Directory, cn=Schemas, cn=Configuration" "objectclass=*" ibm-slapdDbInstance | tail +2 | $AWK -F"=" '{print $2}'`

			# get database user name while at it
			ldapdbuser=`ldapsearch $ldap_cred -s base -b "cn=Directory, cn=RDBM Backends, cn=IBM Directory, cn=Schemas, cn=Configuration" "objectclass=*" ibm-slapdDbUserID | tail +2 | $AWK -F"=" '{print $2}'`

			# get database name while at it
			ldapdbname=`ldapsearch $ldap_cred -s base -b "cn=Directory, cn=RDBM Backends, cn=IBM Directory, cn=Schemas, cn=Configuration" "objectclass=*" ibm-slapdDbName | tail +2 | $AWK -F"=" '{print $2}'`

			# if it was not running before, then stop it again
			if [ $running_status != 0 ];then
				print "Stopping the IDS LDAP server from admin only mode" | $logcmd
				stop_ldap
			fi

		else # Get info from the ldap config file

			print "Obtaining the database instance name" | $logcmd
			echo "\tcat $ibmslapd_conf | \\" | $logcmd
			echo "\t$AWK '{if (toupper(\$1)==toupper(\"ibm-slapdDbInstance:\")){print "\t"\$2}}'" | $logcmd
			cat $ibmslapd_conf | $AWK '{if (toupper($1)==toupper("ibm-slapdDbInstance:")){print "\t"$2}}' | $logcmd
			ldapdbinst=`cat $ibmslapd_conf | $AWK '{if (toupper($1)==toupper("ibm-slapdDbInstance:")){print $2}}'`

			print "Obtaining the database manager user name" | $logcmd
			echo "\tcat $ibmslapd_conf | \\" | $logcmd
			echo "\t$AWK '{if (toupper(\$1)==toupper(\"ibm-slapdDbUserId:\")){print "\t"\$2}}'" | $logcmd
			cat $ibmslapd_conf | $AWK '{if (toupper($1)==toupper("ibm-slapdDbUserId:")){print "\t"$2}}' | $logcmd
			ldapdbuser=`cat $ibmslapd_conf | $AWK '{if (toupper($1)==toupper("ibm-slapdDbUserId:")){print $2}}'`

			print "Obtaining the database name" | $logcmd
			echo "\tcat $ibmslapd_conf | \\" | $logcmd
			echo "\t$AWK '{if (toupper(\$1)==toupper(\"ibm-slapdDbName:\")){print "\t"\$2}}'" | $logcmd
			cat $ibmslapd_conf | $AWK '{if (toupper($1)==toupper("ibm-slapdDbName:")){print "\t"$2}}' | $logcmd
			ldapdbname=`cat $ibmslapd_conf | $AWK '{if (toupper($1)==toupper("ibm-slapdDbName:")){print $2}}'`
		fi

		if [ X$ldapdbinst = X ];then
			print "Error: ibm-slapdDbUserId not found in $ibmslapd_conf file" | $logcmd
			unset ibmslapd_conf
		else
			# db user was good, continue grabbing db2 information

			# Since db2 information is being asked for, assume db2 commands will be issued
			# and source in the instance owners db2 profile

			# source in the instance owners db2 profile
			echo "Sourcing in the the database instance owner's DB2 profile"
			echo "\t. ~$ldapdbinst/sqllib/db2profile" | $logcmd
			. ~$ldapdbinst/sqllib/db2profile

			get_tabschema
		fi
	done
}

get_db2_level(){
	ask_ldapdbinst
	if [ X$db2vers = X ]||[ X$db2fp = X ];then
		echo db2level
		db2level
		db2vers=`db2level | $AWK '{if (p=index($0,"DB2 v")){print substr($0,p+5,1)}}'`
		db2fp=`db2level | grep FixPak | $AWK -F"\"" '{print $(NF-1)}'`
	fi
}

backup_db2(){
	backup_dir=$1

	cat << EOF

The IBM Tivoli Directory Server provides database backup and restore commands
named dbback and dbrestore.  These commands are also known as idsdbback
and idsdbrestore in IDS version 6.  The advantage to using idsdbback and
idsdbrestore is ease of use and the backing up of the
directory schema, configuration file and key stash file, in addition to the
database.  Alternatively, the DB2 backup command can be used to back up the
IBM Directory Server database.  The advantage to using the DB2 backup and
restore utilities is in the restore functionality.  The DB2 restore utility has
greater flexibility in handling differences between the backed up database
and the existing database.

Here are some example differences:
- The db2 restore command can restore a database from a different instance and
  database name.
- The db2 restore command can restore the database to one or more different
  storage system locations.  This allows the database to be spread across
  multiple file systems for better performance.
- The db2 backup and restore commands do not preserve and restore the key stash
  files.  These files can by copied manually or the source and target databases
  can be created with the same key stash seed.

EOF
	print "Press <Enter> to proceed with using the DB2 database backup command" | $logcmd
	read ans

	unset backup_dir_local
	while [ X$backup_dir_local = X ];do
		print -n "Enter the name of a non-existing directory in which to place the backup (default: $backup_dir): "
		read ans
		print "Enter the name of a non-existing directory in which to place the backup (default: $backup_dir): "$ans >> $logfile

		if [ X$ans = X ];then
			ans=$backup_dir
		fi

		if [ -d $ans ] || [ -f $ans ];then
			print "The specified directory in the file system already exists" | $logcmd
		else
			backup_dir=$ans		# this is an output variable to this routine; do not remove
			backup_dir_local=$ans
		fi
	done

	ask_ldapdbinst
	stop_ldap	# stop the ldap server

	# force all connections to the database to be closed
	echo "db2 force applications all" | $logcmd
	db2 force applications all | $logcmd

	# Create the backup directory with the proper ownerships
	echo mkdir $backup_dir 2>&1 | $logcmd
	mkdir $backup_dir 2>&1 | $logcmd
	echo chown $ldapdbinst $backup_dir | $logcmd
	chown $ldapdbinst $backup_dir | $logcmd
	# chgrp dbsysadm $backup_dir | $logcmd
	echo chgrp `groups $ldapdbinst | $AWK '{print $NF}'` $backup_dir | $logcmd 
	chgrp `groups $ldapdbinst | $AWK '{print $NF}'` $backup_dir | $logcmd

	# do the backup
	print $0: Start of the directory server back up : `date` | $logcmd

	echo "db2 backup db $ldapdbname to $backup_dir" | $logcmd
	db2 backup db $ldapdbname to $backup_dir |&

	success_msg_found=0
	read_rc=0
	while [ $read_rc = 0 ];do
		read -p line
		read_rc=$?

		if [ $read_rc = 0 ];then
			print $line | $logcmd
			if [ $success_msg_found = 0 ];then
				success_msg_found=`echo $line | grep "Backup successful." | wc -l`
			fi
		fi
	done
	print $0: End of the directory server back up : `date` | $logcmd

	if [ $success_msg_found = 0 ];then
		backup_db2_retcode=1
		print "" | $logcmd
		print "DB2 backup failed" | $logcmd
	else
		backup_db2_retcode=0
		print "" | $logcmd
		print "DB2 backup suceeded" | $logcmd
	fi
}

ask_db2_restore_dir(){

	unset restore_dir
	while [ X$restore_dir = X ];do
		print -n "Enter the name of the directory containing the db2 backup (no default): "
		read ans
		print "Enter the name of the directory containing the db2 backup to be restored (no default): "$ans >> $logfile

		if [ X$ans != X ];then
			if [ ! -d $ans ];then
				print "The specified name is not a directory in the file system" | $logcmd
			else
				restore_dir=$ans

				# get the backed up database name from the backup filename and verify that is
				# is the same as the database name being worked upon
				backed_up_dbname=`ls -tr $restore_dir | $AWK -F"." '{n=$1}END{print tolower(n)}'`
				if [ "X$backed_up_dbname" != "X`echo $ldapdbname | $AWK '{print tolower($1)}`" ];then
					print "" | $logcmd
					print "The backed up database name ($backed_up_dbname) and the existing database name ($ldapdbname) are different." | $logcmd

				cat << EOF | $logcmd

This is not necessarily a problem.  It just means there are differences in the backed
up and existing databases.  The restore command will try to resolve any differences,
but to be safe make sure there is a back up of the existing database before proceeding.

EOF
					askif "Proceed with the restore" y
					if [ $doit = 0 ];then
						unset restore_dir
					fi
				fi
			fi
		fi
	done
}

check_unix_os_tunings(){

doit=2
while [ $doit = 2 ];do

askif "Check Unix Operating System specific tunes" y
if [ $doit = 1 ];then

	if [ $OS = "SunOS" ];then

		physmem=`prtconf | grep "Memory size" | $AWK '{print $3}'`
		physmem=$(( $physmem * 1024 * 1024 ))

		# Make sure shared memory is set
		sysdef | grep -i shmmax >/dev/null
		if [ $? = 1 ];then
			cat << EOF | $logcmd

A shared memory maximum is not defined.
Add a "set shmsys:shminfo_shmax = <size>" line to the /etc/system file
The following setting is recommended:

set shmsys:shminfo_shmmax = $physmem

Reboot the machine for this change to take affect.

Check failed:  Make the recommened changes and try again.
EOF
			# ask again
			doit=2
		else

			# check the shared memory max setting
			shmmax=`sysdef | grep -i shmmax | $AWK '{print $1}'`

			# If less than 20% difference, then OK
			pdif=$(( shmmax / physmem -1 ))
			if [ $pdif -gt 20 ];then

			cat << EOF | $logcmd

It is recommended that the shared memory maximum be increased.
Modify the "set shmsys:shminfo_shmax = <size>" line in the /etc/system file
The following setting is recommended:

set shmsys:shminfo_shmmax = $physmem

Reboot the machine for this change to take affect.

Check failed:  Make the recommened changes and try again.
EOF
				# ask again
				doit=2
			else
			cat << EOF | $logcmd

Check succeeded: The shared memory maximum is at its recommended setting in the /etc/system file.
EOF
			fi
		fi
		
		if [ `ulimit -d` != unlimited ] || \
			[ `ulimit -v` != unlimited ] || \
			[ `ulimit -f` != unlimited ];then

			cat << EOF | $logcmd
"" | $logcmd
"One or more of process data size, virtual memory size or maximum file" | $logcmd
"size is limited.  These are the current settings:" | $logcmd
"" | $logcmd
EOF
			ulimit -a | $logcmd
			cat << EOF | $logcmd

It is recommended that these be set to unlimited with the following commands:
     ulimit -d unlimited
     ulimit -v unlimited
     ulimit -f unlimited
See the Performance Tuning Guide for more information.

EOF
		else
			cat << EOF | $logcmd
Check succeeded: The operating system resource limits are at their recommended settings.

EOF
		fi
	fi

	if [ $OS = "AIX" ];then

		if [ `ulimit -d` != unlimited ] || \
			[ `ulimit -m` != unlimited ] || \
			[ `ulimit -f` != unlimited ];then

			cat << EOF | $logcmd

One or more of process data size, virtual memory size or maximum file
size is not unlimited.  These are the current settings:

EOF
			ulimit -a | $logcmd
			cat << EOF | $logcmd

It is recommended that these be set to unlimited.  This is done by
updating the "/etc/security/limits" file.  The relevant lines in
this file are as follows:

EOF
			cat /etc/security/limits | $AWK '{
				if (substr($1,length($1))==":"){
					if($1=="default:"||$1=="root:"){
						f=1;print $0
					}else{
						f=0
					}
				}
				if (f){print $0}}' | egrep 'fsize|data|rss' | $logcmd
			cat << EOF | $logcmd

Make updates to either the "default:" or the "root" sections.
Set the values to "-1" to indicate unlimited.  Re-login for the
changes to take affect.

Check failed:  Make the recommened changes and try again.
EOF
			# ask again
			doit=2
		fi

		cat << EOF | $logcmd

Check succeeded: The operating system resource limits are at the recommended settings.

The following information describes some additional AIX resource limits that should
be considered, yet cannot be checked by this script.

1) By default on AIX, the process memory size is limited to 128MB.  AIX supports a
   large memory model which is enabled through the LDR_CNTRL=MAXDATA=<num segments>
   variable.  Refer to the Performance Tuning Guide for more information.

2) The standard file system on AIX has a 2GB file size limit, regardless of the
   ulimit setting.  In order to create a file that is greater than 2GB, the file
   system can either be created as an Enhanced Journaled File Systems, also known
   as JFS2, or it can be created with the Large File Enabled option.  These options
   can be found through the smit inteface.

EOF

		print "Press <Enter> to continue with the Unix Operating System tunes." | $logcmd
		read ans

		print "Checking AIX environment variable tunes." | $logcmd

		check_var(){
			varname=$1
			varvalue=$2

			print "  Checking $varname=$varvalue" | $logcmd

			# check current environment
			print -n "    Looking in the current shell environment: " | $logcmd
			success=0
			env | grep "$varname=$varvalue" >/dev/null
			if [ $? = 0 ];then
				success=1
				print "found." | $logcmd
			else
				print "not found." | $logcmd
			fi
				
			# check /etc/environment
			print -n "    Looking in /etc/environment: " | $logcmd
			cat /etc/environment | grep "$varname=$varvalue" >/dev/null
			if [ $? = 0 ];then
				success=1
				print "found." | $logcmd
			else
				print "not found." | $logcmd
			fi

			# check LDAP config file
			ask_ldap_inst
			print -n "    Looking in $ibmslapd_conf: " | $logcmd
			cat $ibmslapd_conf | $AWK '/cn:.{1,}Front.{1,}End/{
				while ($0!=""){print $0;getline};exit}' | \
				grep "ibm-slapdSetEnv:" | grep "$varname=$varvalue" >/dev/null
			if [ $? = 0 ];then
				success=1
				print "found." | $logcmd
			else
				print "not found." | $logcmd
			fi

			if [ $success = 1 ];then
				print "Check succeeded:  Recommended environment variable tune $varname=$varvalue was found." | $logcmd
			else
				print "Check Failed:  Recommended environment variable tune $varname=$varvalue was not found." | $logcmd
			fi
		}

		check_var SPINLOOPTIME 650
		check_var MALLOCMULTIHEAP 1
	fi

	if [ $OS != "SunOS" ] && [ $OS != "AIX" ];then
		print "" | $logcmd
		print "This script has not been tested with the currently running Operating System ($OS)." | $logcmd
		print "In general, the operating system tunes consist of making the process data" | $logcmd
		print "size, process memory size and maximum file size resources unlimited." | $logcmd
		print "" | $logcmd
		print "On most Unix systems, the \"ulimit -a\" command shows the current limits on" | $logcmd
		print "these and other resources.  Here is the output of \"ulimit -a\" on this system:" | $logcmd
		print "" | $logcmd
		ulimit -a | $logcmd
		print "" | $logcmd
		print "The resources are typically named \"file\", \"data\" and \"memory\" and should" | $logcmd
		print "all be set to unlimited.  Note that the \"ulimit\" command can be used to set these" | $logcmd
		print "limits, but the changes are typically not persistent across shell environments." | $logcmd
		print "" | $logcmd
		print "There is usually some more peristent way to set these limits.  Refer to Operating" | $logcmd
		print "System documentation for more information." | $logcmd
		print "" | $logcmd
	fi
fi
done

}

check_change_log(){

doit=2
while [ $doit = 2 ];do

	askif "Check whether LDAP change log is configured" y
	if [ $doit = 1 ];then
		ask_ldap_cred	# get the ldap admin acccount, also starts ldap

		# Verify LDAP change log is not configured
		if [ `ldapsearch $ldap_cred -s base -b "" "objectclass=*" | \
			grep "CN=CHANGELOG" 2>&1`X != X ];then

			print "The LDAP change log is configured" | $logcmd
			print "For best performance, unconfigure LDAP change log using one of the following commands:" | $logcmd
			print "" | $logcmd
			print "  For IDS version 6.0:" | $logcmd
			print "    idsucfgchglg -I $idsldap_inst" | $logcmd
			print "  For IDS versions lower than 6.0:" | $logcmd
			print "	   ldapucfg -g" | $logcmd
			print "" | $logcmd

			# ask again
			doit=2
		else
			print "" | $logcmd
			cat << EOF | $logcmd
The LDAP change log is not configured.  This is the recommended setting for best
performance.  Only configure the LDAP change log if necessary.

EOF
		fi
	fi
done
}

check_db2_tunings(){

	quesn="Do the DB2 parameter tuning tasks.  This is fairly fast and recommended,\n"
	quesn=$quesn" especially if it has never been done before"
	askif "$quesn" y
	if [ $doit = 1 ];then

		ask_ldapdbinst

		# note the db2_tunings.sh script will stop the LDAP server

		# Do the DB2 tunings
		print "" | $logcmd
		print $0: Run db2_tunings.sh : `date` | $logcmd
		db2_tunings.sh 2>&1 | $logcmd
		print $0: Finished db2_tunings.sh : `date` | $logcmd

		print "Press <Enter> to continue with tuning." | $logcmd
		read ans
	fi
}

check_ldap_desc_index(){

# Check LDAP_DESC(AEID,DEID) index tune

cat << EOF | $logcmd

Creation of the LDAP_DESC(AEID,DEID) index is a tune that primarily improves the
performance of small subtree operations.  The following are some examples:

- Performing a small subtree with an "objectclass=*" filter when 100K or users exist
  in the directory server.
- Placing an ACL on a small subtree when millions of entries exist in the directory
  server, for example, using ldapmodify to place the ACL.
- Using the fixacls.sh tuning guide script.  The performance benefit of the AEID,DEID
  index is at its greatest when the fixacls.sh script is run after a bulkload of a
  relatively small number of LDAP entries into a directory server with millions of
  LDAP entries.  The fixacls.sh is only needed when the standard LDAP data format
  option is selected and 100K or more users are loaded into the directory server
  using bulkload.  Prior to TAM 6.0, the only data format model supported was the
  standard format.  In TAM 6.0, the amldap2v6 utility is provided to migrate an LDIF
  file dumped using the db2ldif command to the new minimal data format.  The minimal
  data format does not require the use of the fixacls.sh tuning guide script after
  a bulkload.

The last two cases are the result of propagating ACLs to the children of a small
subtree.  Propagating ACLs to the childern of a large subtree is slow regardless of
the existence of this index.

Without the LDAP_DESC(AEID,DEID) index all subtree searches using an "objectclass=*"
filter are slow on a large directory server, even if the subtree being searched is
small.  One thing to consider when designing an LDAP search is the order in which the
subtree criteria is used in evaluating the response.  The subtree criteria is used last.
Because of this, substree searches specifying a commonly used objectclass values on a
small subtree of a large directory are slow even if the LDAP_DESC(AEID,DEID) exists.
For best performance, avoid searches specifying a commonly used objectclass value.

If the directory server has very many existing entries, the creation of this index takes
a long time.  One measure is 35 minutes on a directory server with 3 million Access
Manager users.

This index can be dropped later if desired by using the following command:
	db2 drop index $tabschema.LDAP_DESC_DEID

EOF

askif "Check whether the LDAP_DESC(AEID,DEID) index already exists" y
if [ $doit = 1 ];then

	# Check whether the LDAP_DESC(AEID,DEID) index exists
	print "" | $logcmd
	print "Checking for the existence of the LDAP_DESC(AEID,DEID) index" | $logcmd

	ask_ldapdbinst

	echo "db2 connect to $ldapdbname >/dev/null" | $logcmd
	db2 connect to $ldapdbname >/dev/null

	echo "db2 describe indexes for table $tabschema.ldap_desc show detail" | $logcmd
	db2 describe indexes for table $tabschema.ldap_desc show detail | $logcmd
	db2 describe indexes for table $tabschema.ldap_desc show detail | grep "+AEID+DEID" | wc -l >$0.tmp
	desc_idx_exists=`cat $0.tmp`
	rm $0.tmp

	echo "db2 terminate >/dev/null" | $logcmd
	db2 terminate >/dev/null

	if [ $desc_idx_exists = 1 ];then

		print "Check succeeded: The LDAP_DESC(AEID,DEID) index already exists" | $logcmd
		print "No action is required." | $logcmd

	else
		print "Check failed: The LDAP_DESC(AEID,DEID) index does not exists" | $logcmd

		askif "Create the index" y
		if [ $doit = 1 ];then

			print $0: "Start the creation of the LDAP_DESC(AEID,DEID) index" : `date` | $logcmd

			echo "db2 connect to $ldapdbname >/dev/null" | $logcmd
			db2 connect to $ldapdbname >/dev/null

			echo "db2 \"create index $tabschema.LDAP_DESC_DEID on $tabschema.LDAP_DESC(AEID,DEID)\"" | $logcmd
			db2 "create index $tabschema.LDAP_DESC_DEID on $tabschema.LDAP_DESC(AEID,DEID)" | $logcmd

			echo "db2 terminate >/dev/null" | $logcmd
			db2 terminate >/dev/null

			print $0: "Finished creating the LDAP_DESC(AEID,DEID) index" : `date` | $logcmd
		fi
	fi
fi 

}

check_am_schema(){

cat << EOF | $logcmd

If the Tvioli Access Manager schema has never been applied, it must be applied once.
The application of the Tivoli Access Manager schema will produce some error messages,
but they typically can be ignored.  If the schema is applied more than once
additional error messages will be displayed, but they also can be ignored.

EOF
askif "Apply the Tivoli Access Manager schema to the LDAP directory" y
if [ $doit = 1 ];then
	ask_ldap_cred	# get the ldap admin account, also start ldap

	echo ldapadd -c $ldap_cred -f secschema.def | $logcmd
	ldapadd -c $ldap_cred -f secschema.def 2>&1 | $logcmd
fi

}

check_conf_file(){

ask_ldap_inst

changed_suf=0
changed_other_suf=0
add_suf_objs=0

print "" | $logcmd
askif "Configure the secauthority=default suffix to the directory server" y
if [ $doit = 1 ];then

	# With the addition of ldapexop, starting in config mode is not longer necessary.
	# ask_ldap_cred_and_start_config_mode_if_supported
	ask_ldap_cred	# also starts the ldap server

	getsufcmd="ldapsearch $ldap_cred -s base -b \"cn=Directory, cn=RDBM Backends, cn=IBM Directory, cn=Schemas, cn=Configuration\" \"objectclass=*\" ibm-slapdSuffix"

	# Checking whether the secauthority=default suffix already exists
	if [ `eval $getsufcmd 2>/dev/null | grep -i secauthority=default | wc -l` = 0 ];then

		print "Adding the Tivoli Access Manager secauthority=default suffix to the directory server." | $logcmd
		# Do not use idscfgsuf.  It cannot run while the server is running.
		# if [ $ldapvers -ge 600 ];then
		#  
		# 	# stop_ldap
		# 	echo "idscfgsuf -n -I $idsldap_inst -s secauthority=default" | $logcmd
		# 	idscfgsuf -n -I $idsldap_inst -s secauthority=default | $logcmd
		# else
			cat << EOFEOF | $logcmd
cat << EOF | ldapmodify $ldap_cred
dn: cn=Directory, cn=RDBM Backends, cn=IBM Directory, cn=Schemas, cn=Configuration
changetype: modify
add: ibm-slapdSuffix
ibm-slapdSuffix: secauthority=default
EOF
EOFEOF
			cat << EOF | ldapmodify $ldap_cred | $logcmd
dn: cn=Directory, cn=RDBM Backends, cn=IBM Directory, cn=Schemas, cn=Configuration
changetype: modify
add: ibm-slapdSuffix
ibm-slapdSuffix: secauthority=default
EOF
		# fi

		# make the change know to the LDAP server using ldapexop
		echo "ldapexop $ldap_cred -op readconfig -scope single \"cn=Directory,cn=RDBM Backends,cn=IBM Directory,cn=Schemas,cn=Configuration\" ibm-slapdSuffix" | $logcmd
		ldapexop $ldap_cred -op readconfig -scope single "cn=Directory,cn=RDBM Backends,cn=IBM Directory,cn=Schemas,cn=Configuration" ibm-slapdSuffix | $logcmd

		changed_suf=1
	else
		print "The Tivoli Access manager secauthority=default suffix is already configured." | $logcmd
	fi
fi

askif "Configure any other suffix(es) to the directory server" y
if [ $doit = 1 ];then

	# With the addition of ldapexop, starting in config mode is not longer necessary.
	# ask_ldap_cred_and_start_config_mode_if_supported
	ask_ldap_cred	# also starts the ldap server

	getsufcmd="ldapsearch $ldap_cred -s base -b \"cn=Directory, cn=RDBM Backends, cn=IBM Directory, cn=Schemas, cn=Configuration\" \"objectclass=*\" ibm-slapdSuffix | grep ibm-slapdSuffix"

	print "" | $logcmd
	print "The suffix(es) that are currently configured are as follows:" | $logcmd
	print "" | $logcmd
	echo "$getsufcmd" | $logcmd
	print "" | $logcmd
	eval $getsufcmd | $logcmd
	print "" | $logcmd

	cat <<EOF | $logcmd
A name must be chosen for any new suffix(es) that are to be configured.  If the name
is in the format of \"cn=<suffix name>\" where <suffix name> is any text, the
LDAP entry for the suffix can be automatically created using a generic container
objectclass.  This format is not required, but if not followed, the the LDAP entry
for the suffix must be created manually, for example using ldapmodify.

EOF
	doit=1
	while [ $doit = 1 ];do

		print -n "Enter the name of the suffix to configure, for example \"cn=example suffix\": " | $logcmd
		read suf
		print $suf >> $logfile

		# Do not use idscfgsuf.  It cannot run while the server is running.
		# if [ $ldapvers -ge 600 ];then
		# 	stop_ldap
		# 	echo "idscfgsuf -n -I $idsldap_inst -s \"$suf\"" | $logcmd
		# 	idscfgsuf -n -I $idsldap_inst -s "$suf" | $logcmd
		# else
			cat << EOFEOF
cat << EOF | ldapmodify $ldap_cred
dn: cn=Directory, cn=RDBM Backends, cn=IBM Directory, cn=Schemas, cn=Configuration
changetype: modify
add: ibm-slapdSuffix
ibm-slapdSuffix: $suf
EOF
EOFEOF
			cat << EOF | ldapmodify $ldap_cred
dn: cn=Directory, cn=RDBM Backends, cn=IBM Directory, cn=Schemas, cn=Configuration
changetype: modify
add: ibm-slapdSuffix
ibm-slapdSuffix: $suf
EOF
		# fi

		# make the change know to the LDAP server using ldapexop
		echo "ldapexop $ldap_cred -op readconfig -scope single \"cn=Directory,cn=RDBM Backends,cn=IBM Directory,cn=Schemas,cn=Configuration\" ibm-slapdSuffix" | $logcmd
		ldapexop $ldap_cred -op readconfig -scope single "cn=Directory,cn=RDBM Backends,cn=IBM Directory,cn=Schemas,cn=Configuration" ibm-slapdSuffix | $logcmd

		changed_suf=1
		changed_other_suf=1

		askif "Configure another suffix" y
	done

	# if a suffix other than secauthority=default was configured
	# and it is not planned to restore the directory from a backup
	# the suffix ldap objects must be created
	if [ $changed_other_suf = 1 ] && [ $db2_restore_from_backup = 0 ];then

		askif "Is the Directory Server to be loaded from an LDIF data file (e.g. using bulkload)" n
		if [ $doit = 1 ];then

			cat << EOF | $logcmd

If the LDIF data file already contains the definitions of the LDAP entries for
the suffixes, the LDAP entries do not need to be created at this time.  Othwerise,
it is recommended that the LDAP entry for any suffix other than the TAM suffix,
secauthority=default, be created at this time.

EOF
			askif "Create any of the suffix entries at this time" y
			if [ $doit = 1 ];then
				add_suf_objs=1
			fi
		else
			cat << EOF | $logcmd

It is recommended that the LDAP entries for any suffix object other than the
TAM suffix, secauthority=default, be created at this time.

EOF
			add_suf_objs=1
		fi
	fi
fi

doit=0
if [ $add_suf_objs = 0 ]&&[ $db2_restore_from_backup = 0 ];then
#if [ $db2_restore = 0 ];then
	askif "Create an LDAP entry for any suffix other than secauthority=default" y
fi

if [ $add_suf_objs = 1 ]||[ $doit = 1 ];then
#if [ $doit = 1 ];then
	ask_ldap_cred	# also starts the ldap server

	getsufcmdl="ldapsearch -L $ldap_cred -s base -b \"cn=Directory, cn=RDBM Backends, cn=IBM Directory, cn=Schemas, cn=Configuration\" \"objectclass=*\" ibm-slapdSuffix"
	print "$getsufcmdl 2>/dev/null | grep -i ibm-slapdSuffix |" | $logcmd
	print "	grep -vi \"ibm-slapdSuffix: secauthority=default\" |" | $logcmd
	print "	$AWK -F\":\" '{print \$2}'" | $logcmd
	print "" | $logcmd

	# For each suffix object, find those that can be automatically created

	# Use a background pipe
	eval $getsufcmdl 2>/dev/null | grep -i ibm-slapdSuffix |
		grep -vi "ibm-slapdSuffix: secauthority=default" |
		$AWK -F":" '{print $2}' | $AWK '{if (tolower(substr($1,1,3))=="cn="){
			print $0
		}}' |&

	# read from background pipe
	read -p suf
	created=0
	while [ "X$suf" != X ];do

		ldapsearch $ldap_cred -s base -b "$suf" "objectclass=*" dn >/dev/null 2>&1

		# if ldap entry does not exist
		if [ $? = 32 ];then
			created=1

			askif "Create an LDAP entry for $suf using a simple container objectclass" y
			if [ $doit = 1 ];then

				name=`echo $suf | $AWK '{print substr($0,4)}'`

				cat <<EndOfFile | $logcmd
cat <<EOF | ldapadd $ldap_cred
dn: $suf
objectclass: top
objectclass: container
cn: $name
EOF

EndOfFile
				cat <<EOF | ldapadd $ldap_cred | $logcmd
dn: $suf
objectclass: top
objectclass: container
cn: $name
EOF
			fi
		fi

		# read from background pipe
		read -p suf
	done
	if [ $created = 0 ];then
		print "No suffixes were found that could automatically be created.  This could be due not finding any" | $logcmd
		print "suffixes of the form \"cn=<suffix name>\" or because all suffixes of that form already have" | $logcmd
		print "LDAP entries created for them." | $logcmd
	fi
	print "" | $logcmd

	# For each suffix object, find those that cannot be automatically created

	# Use a background pipe
	eval $getsufcmdl 2>/dev/null | grep -i ibm-slapdSuffix |
		grep -vi "ibm-slapdSuffix: secauthority=default" |
		$AWK -F":" '{print $2}' | $AWK '{if (tolower(substr($1,1,3))!="cn="){
			print $0
		}}' |&

	# read from background pipe
	read -p suf
	not_created=0
	while [ "X$suf" != X ];do

		ldapsearch $ldap_cred -s base -b "$suf" "objectclass=*" dn >/dev/null 2>&1

		# if ldap entry does not exist
		if [ $? = 32 ];then
			not_created=1
			print "Could not automatically create LDAP entry for \"$suf\""
		fi
		# read from background pipe
		read -p suf
	done
	if [ $not_created = 1 ];then
		cat <<EndOfFile | $logcmd

The LDAP entry for one or more suffixes requires manual creation, for example
using ldapmodify.  The following is an example of a command to create an LDAP
entry for a suffix with an organization objectclass named "o=ibm":

cat <<EOF | ldapadd $ldap_cred
dn: o=ibm
objectclass: top
objectclass: organization
o: ibm
EOF

EndOfFile

		print "Press <Enter> when all desired suffix entries have been created." | $logcmd
		read ans
	else
		print "No suffixes were found that require manual creation of their LDAP entries." | $logcmd
		print "No action is required." | $logcmd
		print "" | $logcmd
	fi
fi

# end of tune the conf file
}

check_am_configed(){

	cat << EOF | $logcmd

Tivoli Access Manager users can be bulkloaded into the Directory Server without
first configuring Tivoli Access Manager.  In order to do this, a minimum, unconfigured
Tivoli Access Manager LDAP object space must be created on the Directory Server.

EOF
	quesn="Create a minimum, unconfigured Tivoli Access Manager LDAP object space"
	askif "$quesn" y
	if [ $doit = 1 ];then
		ask_ldap_cred	# get the ldap admin account, also start ldap

		cat << EOF | $logcmd

User and group tracking information format

Selecting Minimal requires fewer LDAP objects to maintain
user and group tracking information.
Previous versions of Access Manager will not be supported in the
Minimal environment.

Selecting Standard requires additional LDAP objects to maintain
user and group tracking information.
All versions of Access Manager can participate in the
Standard environment.

Refer to the Tivoli Access Manager Installation Guide for more information
on the LDAP data format selection.

EOF
		askif "Enable Minimal Data Format" y
		if [ $doit = 1 ];then
			data_format_vers=6.0
		else
			data_format_vers=3.0
		fi

		# This if statement allows the return code from the ldapadd to be checked
		# and at the same time the output to be logged.
		if [ 1 = 1 ];then

			echo "cat am_min_unconf.ldif | $AWK -v v=$data_format_vers '{"
			echo "	if (\$1==\"version:\"){print \"version: \"v}"
			echo "	else {print \$0}}' |"
			echo "	ldapadd $ldap_cred -c"

			cat am_min_unconf.ldif | $AWK -v v=$data_format_vers '{
				if ($1=="version:"){print "version: "v}
				else {print $0}}' |
				ldapadd $ldap_cred -c 2>&1

			if [ $? != 0 ];then
				cat << EOF
One or more errors were encountered during the creation of the minimum, unconfigured TAM object
space.  Errors can occur in this step if the TAM object space already exists in the directory
server, e.g. some previously configured or unconfigured TAM object space exists.  The errors can
typically be ignored, since the LDAP objects added in this step will not override those that
already exist in the directory server.
EOF
			fi
		fi | $logcmd
	fi

	print "" | $logcmd
	print "***" $0: `date` | $logcmd
	print "" | $logcmd

	cat <<EOF | $logcmd
Tivoli Access Manager supports a subdomain feature that allows users to be a part
of one or more separately managed domains.

If the domain feature is to be used and users are to be bulkloaded into the
Directory Server before Tivoli Access Manager is configured with domain information,
a minimum, unconfigured domain definition can be created now to make this
possible.

EOF
	askif "Create one or more minimum, unconfigured Tivoli Access Manager domains" y

	doit_save=$doit
	if [ $doit = 1 ];then
		ask_ldap_cred	# get the ldap admin account, also start ldap
		data_format_vers=`ldapsearch -L $ldap_cred -s base -b secauthority=default "objectclass=*" 2>&1 |
			$AWK '{if ($1=="version:"){print $2}}'`
		if [ "X$data_format_vers" = "X6.0" ];then
			ldap_data_format=new
		else
			ldap_data_format=old
		fi
	fi
	doit=$doit_save
	while [ $doit = 1 ];do

		print -n "Enter the name of the domain to create: " | $logcmd
		read dom
		print $dom >> $logfile

		askif "Are you sure you want to create a minimum, unconfigured domain by the name of $dom" n
		if [ $doit = 1 ];then

			# This if statement allows the return code from the ldapadd to be checked
			# and at the same time the output to be logged.
			if [ 1 = 1 ];then

				echo "mk_am_min_unconfig_dom_ldif.sh $dom $ldap_data_format | ldapadd $ldap_cred" | $logcmd
				mk_am_min_unconfig_dom_ldif.sh $dom $ldap_data_format | ldapadd $ldap_cred 2>&1

				if [ $? != 0 ];then
					cat << EOF
One or more errors were encountered during the creation the minimum, unconfigured TAM domain
object space.  Errors can occur in this step if the TAM domain object space already exists in
the directory server.

EOF
				else
					# successful
					cat << EOF | $logcmd
Note that to complete the configuration of the domain, use the following pdadmin command on a machine
with Tivoli Access Manager configured:
	pdadmin -a <tam_admin_id> -p <password> domain create $dom <dom_admin_id> <password> [-desc <description>]

EOF
				fi
			fi | $logcmd
		fi
		askif "Create another minimum, unconfigured Tivoli Access Manager domains" y
	done
}

check_suffix_acls(){

#old: if [ X$add_suf_objs = X ] || [ X$add_suf_objs = X1 ];then
if [ 1 = 1 ];then

	cat << EOF | $logcmd

Checking whether Tivoli Access Manager ACLs are applied to all the LDAP suffixes.  Note
that TAM ACLs are typically applied to LDAP suffixes when TAM is configured, but they
should be applied at this time if a large number of users are to be bulkloaded into the
directory server before TAM is configured.  This will allow bulkload to propagate the
ACLs on suffixes instead of allowing it to occur during TAM configuration.  Propagation
of ACLs to millions of users during TAM configuration will take a long time and could
exceed the DB2 transaction log disk space allocation.

Another situation in which TAM ACLs may not have been applied to all suffixes in the
directory server is when a suffix is defined after TAM has been configured.

EOF
	ask_ldap_cred	# get the ldap admin account, also start ldap

	echo "\tcheck_ldap_acls.sh localhost $ldap_pwd $ldap_binddn $ldap_port 2>&1 | wc -l" | $logcmd
	check_ldap_acls.sh localhost $ldap_pwd $ldap_binddn $ldap_port 2>&1 | wc -l |&
	wait $!
	read -p upds
	print "\t"$upds | $logcmd

	if [ $upds = 0 ];then
		cat << EOF | $logcmd

Check complete: TAM ACLs have already been applied to all configured suffixes.  No action is
required.

EOF
	else
		cat << EOF | $logcmd
Check complete:  TAM ACLs have not been applied to all configured suffixes.

This step may recommend adding aclentry attributes and replacing ownerpropagate entryowner
aclpropagate attributes on suffix LDAP object.  If a custom defined ACL model is being used,
the recommendations may not be appropriate.  Othwerise, allow any recommended updates to be
performed.

The following changes are recommended.

EOF
		echo "check_ldap_acls.sh localhost $ldap_pwd $ldap_binddn $ldap_port 2>&1" | $logcmd
		check_ldap_acls.sh localhost $ldap_pwd $ldap_binddn $ldap_port 2>&1 | $logcmd
		print "" | $logcmd
		askif "Proceed with these modifications" y
		if [ $doit = 1 ];then

			echo "check_ldap_acls.sh localhost $ldap_pwd $ldap_binddn $ldap_port | ldapadd $ldap_cred" | $logcmd
			check_ldap_acls.sh localhost $ldap_pwd $ldap_binddn $ldap_port | \
				ldapadd $ldap_cred 2>&1 | $logcmd
		fi
	fi
fi # end of add_suf_objs check

}

do_runstats(){

cat << EOF | $logcmd

Perform a DB2 runstats.  This is recommended after a large number of updates,
if it has never been done before or after a fresh configuration of the
directory server.  This can take a long time on a large directory, but is
important for good directory performance.

EOF
askif "Do a DB2 runstats" y
while [ $doit = 1 ];do

	doit=0

	# do the runstats
	print $0: Run tam_runstats.sh : `date` | $logcmd
	echo "tam_runstats.sh" | $logcmd
	tam_runstats.sh 2>&1 | $logcmd
	print $0: Finished tam_runstats.sh : `date` | $logcmd

	if [ $? != 0 ];then
		askif "Do a DB2 runstats" y
	else
		performed_runstats=1
	fi
done
}

disable_sysstat_tune(){

print "Disabling any system statistic tunes that may be in effect from"
print "running the sysstat_tune.sh script."
print ""

ask_ldapdbinst
print $0: Running sysstat_tune.sh disable : `date` | $logcmd
sysstat_tune.sh disable 2>&1 | $logcmd
print $0: sysstat_tune.sh disable has completed : `date` | $logcmd

}

check_containers(){

ask_ldapdbinst

for i in `print $1 | $AWK -F"'" '{for (i=2;i<=NF;i+=2){print $i}}'`;do

	print "Checking $i" | $logcmd

	this_container_good=1

	# Make sure the named directory exists
	if [ ! -e $i ];then
		this_container_good=0
		print "Container path error on $i: The directory does not exist" | $logcmd
		continue
	fi

	# Make sure that a directory was specified
	if [ ! -d $i ];then
		this_container_good=0
		print "Container path error on $i: The name specified is not a directory" | $logcmd
		continue
	fi

	# check for root of file system
	if [ `ls $i | grep "lost+found" | wc -l` != 0 ];then
		this_container_good=0
		print "Container path error on $i: The directory is the root of a file system" | $logcmd
		print "  Make a subdirectory in $i and specify that as the container path." | $logcmd
		continue
	fi

	# check for empty directory
	if [ `ls $i | wc -l` != 0 ];then
		this_container_good=0
		print "Container path error on $i: The directory is not empty" | $logcmd

		askif "List the files in $i" y
		if [ $doit = 1 ];then
			echo ls $i | $logcmd
			ls $i | $logcmd
		fi
		askif "Remove the files and directories in $i" y
		if [ $doit = 1 ];then
			askif "This operation cannot be reversed.  Are you sure?" n
			if [ $doit = 1 ];then
				echo rm -r $i/* | $logcmd
				rm -r $i/* | $logcmd

				if [ `ls $i | wc -l` != 0 ];then
					print "Container path error on $i: Container path is still not empty" | $logcmd
				else
					this_container_good=1
				fi
			fi
		fi
	fi

	# check for database ownership of the directory

	if [ `ls -ld $i | $AWK '{print $3}' | grep $ldapdbinst | wc -l` = 0 ];then

		this_container_good=0
		print "Container path error on $i: $ldapdbinst is not the owner of the directory" | $logcmd

		askif "List the directory information" y
		if [ $doit = 1 ];then
			echo ls -ld $i | $logcmd
			ls -ld $i | $logcmd
		fi
		askif "Change directory ownership to $ldapdbinst" y
		if [ $doit = 1 ];then
			echo chown $ldapdbinst $i | $logcmd
			chown $ldapdbinst $i | $logcmd
			if [ `ls -ld $i | $AWK '{print $3}' | grep $ldapdbinst | wc -l` = 0 ];then
				print "Container path error on $i: $ldapdbinst is still not the owner of the directory" | $logcmd
			else
				this_container_good=1
			fi
		fi
	fi

	if [ $this_container_good = 0 ];then
		good_containers=0
	fi
done

}

# get the current container paths
get_tablespace_containers(){

	db2 connect to $ldapdbname 2>&1 >/dev/null
	db2 list tablespace containers for $1 | 
		$AWK -v q="'" '{if ($1=="Name"){
			if (!n){ n=1;print "(path" }else{ print ", path" }
			print q $NF q
		}
		}END{
			if (n){ print ")" }
		}'
	db2 terminate 2>&1 >/dev/null
}

# Note this routine requires the following input variables
# 		tablespace0containers
# 		tablespace1containers
# 		tablespace2containers
# 		tablespace3containers
#		local_restore_dir
restore_db2_redir(){

	tablespace0containers_tmp=`echo $tablespace0containers`
	tablespace1containers_tmp=`echo $tablespace1containers`
	tablespace2containers_tmp=`echo $tablespace2containers`
	tablespace3containers_tmp=`echo $tablespace3containers`

	# The following were the original commands before the "into" clause
	# was added to the restore.
	# db2 restore db $ldapdbname from $local_restore_dir replace existing redirect
	# db2 restore db $ldapdbname continue

	# Use a true conditional in order to log the results and still be able to check the
	# return status on commands
	if [ 1 = 1 ];then

		echo "db2 restore db $backed_up_dbname from $local_restore_dir into $ldapdbname replace existing redirect"
		db2 restore db $backed_up_dbname from $local_restore_dir into $ldapdbname replace existing redirect
		restore_db2_retcode=$?
		if [ $restore_db2_retcode = 0 ]||[ $restore_db2_retcode = 2 ];then
			print $result_msg $restore_db2_retcode
			echo "db2 \"set tablespace containers for 0 using $tablespace0containers_tmp\""
			db2 "set tablespace containers for 0 using $tablespace0containers_tmp"
			restore_db2_retcode=$?
		fi
		if [ $restore_db2_retcode = 0 ];then
			print $result_msg $restore_db2_retcode
			echo "db2 \"set tablespace containers for 1 using $tablespace1containers_tmp\""
			db2 "set tablespace containers for 1 using $tablespace1containers_tmp"
			restore_db2_retcode=$?
		fi
		if [ $restore_db2_retcode = 0 ];then
			print $result_msg $restore_db2_retcode
			echo "db2 \"set tablespace containers for 2 using $tablespace2containers_tmp\""
			db2 "set tablespace containers for 2 using $tablespace2containers_tmp"
			restore_db2_retcode=$?
		fi
		if [ $restore_db2_retcode = 0 ];then
			print $result_msg $restore_db2_retcode
			echo "db2 \"set tablespace containers for 3 using $tablespace3containers_tmp\""
			db2 "set tablespace containers for 3 using $tablespace3containers_tmp"
			restore_db2_retcode=$?
		fi
		if [ $restore_db2_retcode = 0 ];then
			print $result_msg $restore_db2_retcode
			echo "db2 restore db $backed_up_dbname continue"
			db2 restore db $backed_up_dbname continue
			restore_db2_retcode=$?
		fi
		print $result_msg $restore_db2_retcode

	fi 2>&1 | $logcmd
}

restore_db2_simple(){

	# Use a true conditional in order to log the results and still be able to check the
	# return status on commands
	if [ 1 = 1 ];then

		echo "db2 restore db $ldapdbname from $local_restore_dir replace existing"
		db2 restore db $ldapdbname from $local_restore_dir replace existing
		restore_db2_retcode=$?
		print $result_msg $restore_db2_retcode

	fi 2>&1 | $logcmd
}

# Note this routine requires these variables if called with the redir option
# 		tablespace0containers
# 		tablespace1containers
# 		tablespace2containers
# 		tablespace3containers
# This routine requires this variable
#		local_restore_dir
# This routine requires one input argument of the following valures:
# 		redir - indicates a redirected restore
#		simple - indicates a simple restore
restore_db2(){

	restore_db2_retcode=0

	stop_ldap	# stop the ldap server

	# force all connections to the database to be closed
	echo "db2 force applications all" | $logcmd
	db2 force applications all | $logcmd

	result_msg="$0 db2 return code:"

	print $0: Start of the directory server restore : `date` | $logcmd
	if [ "X$1" = Xredir ];then
		restore_db2_redir
	else
		restore_db2_simple
	fi
	print $0: End of the directory server restore : `date` | $logcmd

	if [ $restore_db2_retcode != 0 ];then
		print "" | $logcmd
		print "DB2 restore failed" | $logcmd
	else
		print "" | $logcmd
		print "DB2 restore suceeded" | $logcmd
	fi
}

check_distribute_and_restore(){

ask_ldapdbinst

cat << EOF | $logcmd

Distributing the DB2 database across multiple disk drives and/or file systems
is a way to increase the available disk space for the DB2 database and is
a way to improve database performance by increasing disk concurrency.

EOF
askif "Distribute the database" n
if [ $doit = 1 ];then

	cat << EOF | $logcmd

The directories in which to distribute the database are defined by variables
within this script, $0.  In order to change the directory names, exit this
script, edit it, search for the variable names listed below and change the
settings of those variables to the desired directory name(s).

The names of the variables and their current settings are listed below:

EOF
	if [ "X$tablespace0containers" != X ];then
		print "     variable name: tablespace0containers"
		print "     setting: "$tablespace0containers
		print ""
	fi
	if [ "X$tablespace1containers" != X ];then
		print "     variable name: tablespace1containers"
		print "     setting: "$tablespace1containers
		print ""
	fi
	if [ "X$tablespace2containers" != X ];then
		print "     variable name: tablespace2containers"
		print "     setting: "$tablespace2containers
		print ""
	fi
	if [ "X$tablespace3containers" != X ];then
		print "     variable name: tablespace3containers"
		print "     setting: "$tablespace3containers
		print ""
	fi
	print "Press <Enter> to proceed with these definitions" | $logcmd
	read ans

	cat << EOF | $logcmd

If the directories being used to distribute the database have never been
checked, failures can occur on the restore.  These validity checks can be
performed at this time.  In some cases, the validity checks are more
stringent than necessary, so an option is provided to skip them.

EOF
	askif "Perform the validity checks on the directories to be used" y
	if [ $doit = 1 ];then

		good_containers=0
		while [ $good_containers = 0 ];do

			print "Checking container paths" | $logcmd
			good_containers=1
			if [ "X$tablespace0containers" != X ];then
				check_containers "$tablespace0containers"
			fi
			if [ "X$tablespace1containers" != X ];then
				check_containers "$tablespace1containers"
			fi
			if [ "X$tablespace2containers" != X ];then
				check_containers "$tablespace2containers"
			fi
			if [ "X$tablespace3containers" != X ];then
				check_containers "$tablespace3containers"
			fi
			if [ $good_containers = 1 ];then
				print "Check succeeded: Container paths are good" | $logcmd
			else
				print "" | $logcmd
				print "Check failed: Container path errors were encountered" | $logcmd
				print "" | $logcmd
			fi
		done
	fi

	# get the table space containers from DB2 for those tablespace not specified in variables
	if [ "X$tablespace0containers" = X ];then
		tablespace0containers=`get_tablespace_containers 0`
	fi
	if [ "X$tablespace1containers" = X ];then
		tablespace1containers=`get_tablespace_containers 1`
	fi
	if [ "X$tablespace2containers" = X ];then
		tablespace2containers=`get_tablespace_containers 2`
	fi
	if [ "X$tablespace3containers" = X ];then
		tablespace3containers=`get_tablespace_containers 3`
	fi

	if [ $db2_restore_from_backup = 1 ];then

		# do not backup existing directory server

		# ask for the name of directory containing the backup to be restored

		restore_db2_retcode=1
		while [ $restore_db2_retcode = 1 ];do

			ask_db2_restore_dir
			local_restore_dir=$restore_dir

			print $0: Doing a redirected restore to spread the database across multiple disk drives : `date` | $logcmd

			# the tablespce*containers variables are input to the following routine
			restore_db2 redir 

		done
	else
		# backup the existing directory if not already done and then do the redirected restore

		if [ X$backup_dir != X ];then

			# already backed up
			# use the previously specified backup directory
			local_restore_dir=$backup_dir	# get the chosen backup directory
			backup_dir_cont=$backup_dir

			print $0: Doing a redirected restore to spread the database across multiple disk drives : `date` | $logcmd

			# the tablespce*containers variables are input to the following routine
			backed_up_dbname=$ldapdbname
			restore_db2 redir 

			if [ $restore_db2_retcode = 1 ];then
				exit 1
			fi
		else

			# backup to a temporary directory

			print "The first step is to back up the existing directory" | $logcmd
			print "The directory chosen for backup is temporary and will be delete" | $logcmd
			print "if the restore is successful" | $logcmd

			backup_db2_retcode=1
			while [ $backup_db2_retcode = 1 ];do

				backup_db2 $backup_dir_cont
				local_restore_dir=$backup_dir	# get the chosen backup directory
				backup_dir_cont=$backup_dir
			done

			print $0: Doing a redirected restore to spread the database across multiple disk drives : `date` | $logcmd

			# the tablespce*containers variables are input to the following routine
			backed_up_dbname=$ldapdbname
			restore_db2 redir 

			if [ $restore_db2_retcode = 1 ];then
				exit 1
			fi

			if [ $db2_restore_from_backup = 0 ];then
				# remove the temporary backup
				rm -r $backup_dir
			fi

		fi
	fi
else
	# else, not distributing across multiple directories

	if [ $db2_restore_from_backup = 1 ];then

		# yet, do want to restore from a back up

		cat << EOF

The IBM Tivoli Directory Server provides database backup and restore commands
named dbback and dbrestore.  These commands are also known as idsdbback
and idsdbrestore in IDS version 6.  The advantage to using idsdbback and
idsdbrestore is ease of use and the backing up of the
directory schema, configuration file and key stash file, in addition to the
database.  Alternatively, the DB2 restore command can be used to restore the
IBM Directory Server database.  The advantage to using the DB2 backup and
restore utilities is greater flexibility in handling differences between the
backed up database and the existing database.

Here are some example differences:
- The db2 restore command can restore a database from a different instance and
  database name.
- The db2 restore command can restore the database to one or more different
  storage system locations.  This allows the database to be spread across
  multiple file systems for better performance.
- The db2 backup and restore commands do not preserve and restore the key stash
  files.  These files can by copied manually or the source and target databases
  can be created with the same key stash seed.

EOF
		print "Press <Enter> to proceed with using the DB2 database restore command" | $logcmd
		read ans

		cat << EOF | $logcmd

The IDS LDAP DB2 database consists of multiple tablespaces, each of which resides
in a different file system "container".  The location of these containers is
defined both in the backed up database and in the existing database that is to be
overwritten.

EOF
		askif "Use the database containers defined in the existing database (not those from the back up)" y
		if [ $doit = 1 ];then

			# Use the containers defined in the existing database

			restore_db2_retcode=1
			while [ $restore_db2_retcode = 1 ];do

				ask_db2_restore_dir
				local_restore_dir=$restore_dir

				tablespace0containers=`get_tablespace_containers 0`
				tablespace1containers=`get_tablespace_containers 1`
				tablespace2containers=`get_tablespace_containers 2`
				tablespace3containers=`get_tablespace_containers 3`

				# the tablespce*containers variables are input to the following routine
				restore_db2 redir 
			done
		else

			# Use the containers defined in the backed up database

			restore_db2_retcode=1
			while [ $restore_db2_retcode = 1 ];do

				ask_db2_restore_dir
				local_restore_dir=$restore_dir

				restore_db2 simple

			done
		fi
	fi
fi

}

# Main script starts here

if [ X"$1" != Ximport ];then

print "" | $logcmd
print $0: "*** Beginning of tuning session ***" : `date` | $logcmd
cat << EOF | $logcmd

Note that this tuning session can be terminated at any prompt by pressing
the Ctrl-C sequence.  To continue tuning where left off, restart the
script and select "y" or "n" to skip the tuning tasks that have already
been performed.

Select an action to be performed:
1 - Tune all components of the directory server
2 - Tune the DB2 system statistics, recommended after a large number of updates
    have been made to the Directory Server, for example, after using the
    bulkload utility.
3 - Distribute the database across multiple disks either by temporarily backing up the
    existing database or restoring from a previous backup.
    This option can also be used to do a simple DB2 restore from a previous backup.

EOF

q="Enter your selection (1, 2 or 3): "
tuning_selection=0
while [ $tuning_selection = 0 ];do
	print -n "$q"
	read ans
	print $q$ans >> $logfile

	if [ X$ans = X1 ] || [ X$ans = X2 ] || [ X$ans = X3 ];then
		tuning_selection=$ans
	fi
done

ask_ldapdbinst

performed_runstats=0

# initialize the db2 restore indicator
db2_restore_from_backup=0
if [ $tuning_selection = 1 ] || [ $tuning_selection = 3 ];then
	# ask about DB2 restore
	askif "Is the Directory Server to be restored from a previous db2 back up" n
	if [ $doit = 1 ];then
		db2_restore_from_backup=1
	else
		askif "Back up the LDAP DB2 database (recommended)" y
		if [ $doit = 1 ];then
			backup_db2_retcode=1
			while [ $backup_db2_retcode = 1 ];do
				backup_db2 $backup_dir_before
			done
		fi
		check_distribute_and_restore
	fi
fi

if [ $tuning_selection = 2 ];then
	# 3 - tune after large number of updates

	askif "Back up the LDAP DB2 database (recommended)" y
	if [ $doit = 1 ];then
		backup_db2_retcode=1
		while [ $backup_db2_retcode = 1 ];do
			backup_db2 $backup_dir_before
		done
	fi
fi

if [ $tuning_selection = 1 ] && [ $db2_restore_from_backup = 0 ];then
	# 1 - Do the full set of performance tuning checks

	# The tabschema being null is a good way to determine that the
	# directory server has not ever been started.
	if [ X$tabschema = X ];then

		print "The Directory Server has never been started." | $logcmd
		print "Press <Enter> to start the Directory Server now." | $logcmd
		read ans

		start_ldap

		get_tabschema
	fi
fi

if [ $db2_restore_from_backup = 1 ];then

	check_conf_file

	check_am_schema

	check_distribute_and_restore

	get_tabschema

	# The tabschema being null is a good way to determine that the
	# directory server has not ever been started.
	if [ X$tabschema = X ];then

		print "The Directory Server has never been started." | $logcmd
		print "Press <Enter> to start the Directory Server now." | $logcmd
		read ans

		start_ldap

		get_tabschema
	fi

fi

if [ $tuning_selection = 1 ];then
	# 1 - Do the full set of performance tuning checks

	check_unix_os_tunings
fi

if [ $tuning_selection = 1 ] || [ $db2_restore_from_backup = 1 ];then

	check_db2_tunings

	check_ldap_desc_index
fi

if [ $tuning_selection = 1 ];then

	# 1 - Do the full set of performance tuning checks
	# and not doing a db2 restore

	check_change_log

	if [ $db2_restore_from_backup = 0 ];then
		check_conf_file

		check_am_schema
	fi

	check_am_configed

	check_suffix_acls
fi

if [ $tuning_selection = 1 ] || [ $tuning_selection = 2 ] || [ $tuning_selection = 3 ];then
	askif "Check for missing or extra DB2 indexes" y
	if [ $doit = 1 ];then
		# Check for DB2 indexes required for good LDAP performance
		print $0: Run check_db2_indexes.sh : `date` | $logcmd
		echo check_db2_indexes.sh | $logcmd
		check_db2_indexes.sh 2>&1 | $logcmd
		print $0: Finished check_db2_indexes.sh : `date` | $logcmd
	fi

	do_runstats
fi

# No longer need to prepare the directory for bulkload.  The important tunes for
# fixacls were added to the fixacls script.

#if [ $tuning_selection = x ];then
if [ 0 = 1 ];then

if [ $db2_restore_from_backup = 0 ];then

	cat << EOF | $logcmd
The directory server is prepared for bulkload.

Please consider the following important tuning information regarding directory
load performance:

If the directory server version is 5.2 or lower and more than 100K TAM users
are to be bulkloaded into the directory server, the bulkload must be performed
with ACL processing disabled for performance reasons.  If ACLs are desired, the
ACL definitions for TAM user objects can be added using the
fixacls.sh script.  The non-TAM user ACLs can be extracted
from the LDIF using the extract_acls.sh tuning guide script and added to the
directory using ldapmodify.

If a bulkload is to be performed on a directory server that already has 100K or
more entries, the existence of the LDAP_DESC(AEID,DEID) index will improve the
propagation of ACLs that occurs during the TAM
fixacls.sh tuning guide script.  The percentage improvement
grows in direct proportion to the size of the existing directory and in inverse
proportion to the number of entries being loaded.

For the LDAP_DESC(AEID,DEID) index to be effective in improving the performance
the propagation of ACLs, the DB2 system statistic table tunes must be disabled.
The DB2 system statistic table tunes can become re-enabled by starting directory
server without the LDAP_MAXCARD=NO environment variable defined.  If the directory
server must be started before performing a bulkload or running the
fixacls.sh script, use the LDAP_MAXCARD=NO environment
variable, for example, as in the following:
	LDAP_MAXCARD=NO ibmslapd
The system statistic table tunes are disabled during this tuning session, but
can also be disabled using the sysstat_tune.sh tuning guide script as in the
example below:
	sysstat_tune.sh disable
and by performing a db2 runstats or reorgchk, the slowest methods.

The disabling of the system statistic table tunes is only effective if the
the directory server is not running or is running with the with the
LDAP_MAXCARD=NO environment variable defined.  Otherwise, the directory server
periodically re-enables a subset of the statistic tunes.

When the load is complete, re-run the this script and select the option to
do the performance tuning checks recommended after a large number of updates
have been made to the Directory Server.

EOF
	print $0: End : `date` | $logcmd
	exit
fi

fi

if [ $performed_runstats = 0 ];then
	quesn="Do the DB2 statistics tunes (sysstat_tune.sh).  This is recommended after\n"
	quesn=$quesn" any DB2 runstats or reorgchk"
	askif "$quesn" y
	if [ $doit = 1 ];then

		print $0: Running the sysstat_tune.sh script : `date` | $logcmd
		sysstat_tune.sh 2>&1 | $logcmd
		print $0: sysstat_tune.sh has completed : `date` | $logcmd
	fi
fi

# potentially add the test_registry script here.

askif "Back up the LDAP DB2 database to save the state of all tunes (recommended)" y
if [ $doit = 1 ];then

	backup_db2_retcode=1
	while [ $backup_db2_retcode = 1 ];do
		backup_db2 $backup_dir_after
	done

fi

# Leave with LDAP started
start_ldap	# start the ldap server

print $0: End : `date` | $logcmd

fi # endif for import
