#!/bin/ksh

# script to determine whether all tables that exist have an index on eid and any
# other index important to IDS

# Restrictions:
# This script must be run under the context of the ldap db2 instance ownwer.  It
# requires write authority to the current directory for temporary
# files.

if [ `uname` = "SunOS" ];then
	AWK=nawk
else
	AWK=awk
fi

# Find the am_tune_ldap.sh script
# If it is in the current path, set command path to null
which am_tune_ldap.sh >/dev/null 2>&1
if [ $? = 0 ];then
	am_tune_ldap_cmd=am_tune_ldap.sh
else
	# not in current command path; try to get the path from the command
	cmdpath=`echo $0 | $AWK -F"/" '{for (i=1;i<NF;i++){p=p""$i"/"}}END{print p}'`
	am_tune_ldap_cmd=$cmdpath""am_tune_ldap.sh

	# verify the path to am_tune_ldap.sh
	which $am_tune_ldap_cmd >/dev/null 2>&1
	if [ $? != 0 ];then
		print "$0 Error: Cannot find am_tune_ldap.sh"
		exit 1
	fi
fi

# import the am_tune_ldap.sh script
detailed_logging=0
. $am_tune_ldap_cmd import

ask_ldapdbinst

print "db2 connect to $ldapdbname"
db2 connect to $ldapdbname

print "Checking whether the LDAP_ENTRY_PEID index allows reverse table scans"

# Use db2look to determine reverse scan definition.

print "\tdb2look -e -d $ldapdbname -u $ldapdbuser -t LDAP_ENTRY 2>&1 | \n\
	grep -v LDAP_ENTRY_PEID2 | \n\
	awk '{if (index(\$0,\"LDAP_ENTRY_PEID\")){ \n\
		print \$0;getline;print \$0;getline;print \$0} \n\
	}' | grep \"ALLOW REVERSE SCANS\" | wc -l"
db2look -e -d $ldapdbname -u $ldapdbuser -t LDAP_ENTRY 2>&1 | \
	grep -v LDAP_ENTRY_PEID2 | \
	awk '{if (index($0,"LDAP_ENTRY_PEID")){ \
		print $0;getline;print $0;getline;print $0} \
	}' | grep "ALLOW REVERSE SCANS" | wc -l |&

read -p allow_reverse_scans
print "\t"$allow_reverse_scans

if [ $allow_reverse_scans = 0 ];then

	# It not defined as a reverse scans.  Here are the commands to fix it.

	cat << EOF

The LDAP_ENTRY_PEID index does not allow reverse table scans
   Recommend recovery:
       su - $ldapdbuser
       . ~$ldapdbinst/sqllib/db2profile
       db2 connect to $ldapdbname
       db2 drop index $tabschema.ldap_entry_peid
       db2 "create index $tabschema.ldap_entry_peid on $tabschema.ldap_entry (EID,PEID) allow reverse scans"
       db2 runstats on table $tabschema.ldap_entry with distribution and detailed indexes all

Note that it could take a few minutes to recreate this index.  For example, on
an average speed machine with 4 million LDAP entries, it takes about 4 minutes.

EOF
	askif "Perform the recommended recovery action" y
	if [ $doit = 1 ];then
		db2 drop index $tabschema.ldap_entry_peid
		# The time for the following command was 1.7 minutes with 4 million ldap entries
		db2 "create index $tabschema.ldap_entry_peid on $tabschema.ldap_entry (EID,PEID) allow reverse scans"
		# the time for the following command was 2.0 minutes with 4 million ldap entries
		db2 runstats on table $tabschema.ldap_entry with distribution and detailed indexes all
	fi
else
	print ""
	print "The LDAP_ENTRY_PEID index is already defined to allow reverse table scans.  No action is required."
fi
print ""

# SECDN is removed, as an easy way to resolve SECDN versus SECDN_T variation
# SECDN:RSECDN:+RSECDN_T+EID:D
# SECDN:SECDN:+SECDN_T+EID:D

# initialize the list of needed indexes

cat << EOF | sort >noneid_idxs_needed.tmp
ALIASEDOBJECT:ALIASEDOBJECT:+ALIASEDOBJECT_T+EID:D
ALIASEDOBJECT:RALIASEDOBJECT:+RALIASEDOBJECT_T+EID:D
CN:CN:+CN_T+EID:D
CN:RCN:+RCN_T+EID:D
DESCRIPTION:DESCRIPTION:+DESCRIPTION_T+EID:D
DESCRIPTION:RDESCRIPTION:+RDESCRIPTION_T+EID:D
LDAP_DESC:LDAP_DESC_AEID:+DEID+AEID:D
LDAP_DESC:LDAP_DESC_DEID:+AEID+DEID:D
LDAP_ENTRY:LDAP_ENTRY_PEID2:+PEID:D
LDAP_ENTRY:LDAP_ENTRY_PEID:+EID+PEID:D
LDAP_ENTRY:LDAP_ENTRY_TRUNC:+DN_TRUNC:D
LDAP_GRP_DESC:LDAP_GRP_DESC_AEID:+DEID+AEID:D
MEMBER:MEMBER:+MEMBER_T+EID:U
MEMBER:RMEMBER:+RMEMBER_T+EID:D
OBJECTCLASS:OBJECTCLASS:+OBJECTCLASS+EID:D
OBJECTCLASS:ROBJECTCLASS:+ROBJECTCLASS+EID:D
PRINCIPALNAME:PRINCIPALNAME:+PRINCIPALNAME_T+EID:D
PRINCIPALNAME:RPRINCIPALNAME:+RPRINCIPALNAME_T+EID:D
SECAUTHORITY:RSECAUTHORITY:+RSECAUTHORITY+EID:D
SECAUTHORITY:SECAUTHORITY:+SECAUTHORITY+EID:D
SECUUID:RSECUUID:+RSECUUID+EID:D
SECUUID:SECUUID:+SECUUID+EID:D
SN:RSN:+RSN+EID:D
SN:SN:+SN+EID:D
SYS:RSYS:+RSYS_T+EID:D
SYS:SYS:+SYS_T+EID:D
TARGETSERVICE:RTARGETSERVICE:+RTARGETSERVICE_T+EID:D
TARGETSERVICE:TARGETSERVICE:+TARGETSERVICE_T+EID:D
TELEPHONENUMBER:RTELEPHONENUMBER:+RTELEPHONENUMBER+EID:D
TELEPHONENUMBER:TELEPHONENUMBER:+TELEPHONENUMBER+EID:D
TSNAME:RTSNAME:+RTSNAME+EID:D
TSNAME:TSNAME:+TSNAME+EID:D
TSTYPE:RTSTYPE:+RTSTYPE+EID:D
TSTYPE:TSTYPE:+TSTYPE+EID:D
UID:RUID:+RUID_T+EID:D
UID:UID:+UID_T+EID:D
UNIQUEMEMBER:RUNIQUEMEMBER:+RUNIQUEMEMBER_T+EID:D
UNIQUEMEMBER:UNIQUEMEMBER:+UNIQUEMEMBER_T+EID:U
ACLPERM:ACLDN_T_INDEX:+ACLDN_TRUNC:D
ENTRYOWNER:OWNERDN_T_INDEX:+OWNERDN_TRUNC:D
POSTALADDRESS:POSTALADDRESS:+POSTALADDRESS_T+EID:D
POSTALADDRESS:RPOSTALADDRESS:+RPOSTALADDRESS_T+EID:D
IBMENTRYUUID:IBMENTRYUUID:+IBMENTRYUUID+EID:D
IBMENTRYUUID:RIBMENTRYUUID:+RIBMENTRYUUID+EID:D
SECDOMAINID:RSECDOMAINID:+RSECDOMAINID_T+EID:D
SECDOMAINID:SECDOMAINID:+SECDOMAINID_T+EID:D
EOF

print "Finding all defined indexes"

echo "db2 \"list tables for schema $tabschema\""
db2 "list tables for schema $tabschema" | grep $tabschema | $AWK '{print $1}' | sort >all_tables.tmp

rm -f eid_idxs.tmp
rm -f noneid_idxs.tmp
for i in `cat all_tables.tmp`;do

	db2 describe indexes for table $tabschema.$i show detail | grep $tabschema | \
		$AWK -v tbl=$i '{
			if ($5 == "+EID" && $3 == "D"){
				fn = "eid_idxs.tmp"
			} else {
				fn = "noneid_idxs.tmp"
			}
			print tbl":"$2":"$5":"$3 >> fn
			#print tbl >> "all_tables.tmp"
		}'
done


# Determine whether there are any missing eid indexes

print "Checking for missing EID indexes"

sort eid_idxs.tmp >eid_idxs_sorted.tmp
mv eid_idxs_sorted.tmp eid_idxs.tmp

# create the set of tables that have an eid index
cat eid_idxs.tmp | $AWK -F ":" '{print $1}' | sort -u >eid_idx_tables.tmp

# SECDN is removed, as an easy way to resolve SECDN versus SECDN_T variation

diff all_tables.tmp eid_idx_tables.tmp | grep "<" | $AWK '{
	if ($2 != "CHANGE" &&
		$2 != "SECDN" &&
		$2 != "RSECDN" &&
		$2 != "SECDN_T" &&
		$2 != "RSECDN_T" &&
		$2 != "LDAP_DESC" &&
		$2 != "LDAP_ENTRY" &&
		$2 != "LDAP_NEXT_EID" &&
		$2 != "PROGRESS" &&
		$2 != "REGISTER" &&
		$2 != "EXPLAIN_ARGUMENT" &&
		$2 != "EXPLAIN_OBJECT" &&
		$2 != "EXPLAIN_OPERATOR" &&
		$2 != "EXPLAIN_PREDICATE" &&
		$2 != "EXPLAIN_STATEMENT" &&
		$2 != "EXPLAIN_STREAM" &&
		$2 != "EXPLAIN_INSTANCE"){
			print $2
	}
}' >potential_missing.tmp

found=0
for i in `cat potential_missing.tmp`;do

	db2 "declare c cursor with hold for select * from $tabschema.$i" >/dev/null
	db2 "open c" >/dev/null
	db2 fetch c for 10 row | grep selected | \
		$AWK '{if ($1 == 10){print 1}else{print 0}}' >hasrow.tmp
	hasrow=`cat hasrow.tmp`
	rm -f hasrow.tmp
	db2 "close c" >/dev/null

	if [ $hasrow = 1 ];then
		found=1
		print "Missing index: $i""I on $tbl(eid)"
		print "   Recommend recovery:"
		print "       su - $ldapdbuser"
		print "       . ~$ldapdbinst/sqllib/db2profile"
		print "       db2 connect to $ldapdbname"
		print "       db2 \"create index $tabschema.$i""I on $tabschema.$i(eid)\""
		print "       db2 runstats on table $tabschema.$i with distribution and detailed indexes all"

		askif "Perform the recommended recovery action" y
		if [ $doit = 1 ];then
			db2 "create index $tabschema.$i""I on $tabschema.$i(eid)"
			db2 runstats on table $tabschema.$i with distribution and detailed indexes all
		fi
	fi
done

if [ $found = 0 ];then
	print "  No missing EID indexes found"
fi

rm -f potential_missing.tmp
rm -f eid_idxs.tmp
rm -f eid_idx_tables.tmp

# Determine whether there are any missing non-eid indexes
print "Checking for missing non-EID indexes"

sort noneid_idxs.tmp >noneid_idxs_sorted.tmp
mv noneid_idxs_sorted.tmp noneid_idxs.tmp

# create lists with index names removed for comparison purposes
cat noneid_idxs_needed.tmp | $AWK -F ":" '{print $1":"$3":"$4}' | sort >nei_needed_short.tmp
cat noneid_idxs.tmp | $AWK -F ":" '{print $1":"$3":"$4}' | sort >nei_short.tmp

diff nei_needed_short.tmp nei_short.tmp | grep "<" | \
	$AWK '{print $2}' >potential_missing.tmp

found=0
for i in `cat potential_missing.tmp`;do

	tbl=`print $i | $AWK -F ":" '{print $1}'`

	# Determine whether the table is even defined in this database
	tbl_defined=`cat all_tables.tmp | $AWK -v t=$tbl '{if ($1==t){print $1}}'`
	if [ "X$tbl_defined" != "X" ];then

		# Determine whether the table has any data

		db2 "declare c cursor with hold for select * from $tabschema.$tbl" >/dev/null
		db2 "open c" >/dev/null
		db2 fetch c for 1 row | grep selected | $AWK '{print $1}' >hasrow.tmp
		hasrow=`cat hasrow.tmp`
		rm -f hasrow.tmp
		db2 "close c" >/dev/null

		if [ $hasrow = 1 ];then

			indx_def_coded=`print $i | $AWK -F ":" '{print $2}'`

			# allow variation on the LDAP_DESC index
			if [ $tbl != "LDAP_DESC" ] || [ $indx_def_coded != "+AEID+DEID" ];then

				found=1

				indx_name=`cat noneid_idxs_needed.tmp | \
					$AWK -F ":" -v idx_def=$indx_def_coded \
						-v tbl=$tbl '{
						if ($1 == tbl && $3 == idx_def){
							print $2
							exit
						}
					}'`

				indx_def=`print $indx_def_coded | $AWK -F "+" '{
					for (i=2;i<NF;i++){
						idx = idx""$i","
					}
					idx = idx""$NF
					print idx
				}'`

				uniq_flag=`print $i | $AWK -F ":" '{print $3}'`
				if [ $uniq_flag = "U" ];then
					uniq_flag=unique
				else
					unset uniq_flag
				fi

				# determine if the index is defined differently than what is expected
				current_indx_def_coded=`cat noneid_idxs.tmp | \
					$AWK -F ":" -v idx_name=$indx_name -v tbl=$tbl '{\
						if ($1 == tbl && $2 == idx_name){
							print $3
							exit
						}
					}'`
				if [ X$current_indx_def_coded != X ];then
					# The index is defined differently

					current_indx_def=`print $current_indx_def_coded | $AWK -F "+" '{
						for (i=2;i<NF;i++){
							idx = idx""$i","
						}
						idx = idx""$NF
						print idx
					}'`

					# remove it from the list of current non-eid indexes
					cat nei_short.tmp | $AWK -F ":" -v tbl=$tbl \
						-v idx_coded=$current_indx_def_coded '{
						if ($1 != tbl || $2 != idx_coded){
							print $0
						}
					}' >nei_short_tmp.tmp
					mv nei_short_tmp.tmp nei_short.tmp

					cat << EOF
Index is incorrect: $indx_name
   Current definition: $tbl($current_indx_def)
   Correct definition: $tbl($indx_def)
   Recommend recovery:
       su - $ldapdbuser
       . ~$ldapdbinst/sqllib/db2profile
       db2 connect to $ldapdbname
       db2 drop $tabschema.index $indx_name
       db2 create "$uniq_flag index $tabschema.$indx_name on $tabschema.$tbl($indx_def)"
       db2 runstats on table $tabschema.$tbl with distribution and detailed indexes all

Note that it could take several minutes to create an index.  For example, on
an average speed machine with 4 million LDAP entries, it takes about 40 minutes
to create the objectclass(objectclass,eid) index.

EOF
					askif "Perform the recommended recovery action" y
					if [ $doit = 1 ];then
						db2 drop index $tabschema.$indx_name
						db2 create "$uniq_flag index $tabschema.$indx_name on $tabschema.$tbl($indx_def)"
						db2 runstats on table $tabschema.$tbl with distribution and detailed indexes all
					fi
				else
					# The index is missing
					cat << EOF
Missing index: $indx_name on $tbl($indx_def)
   Recommend recovery:
       su - $ldapdbuser
       . ~$ldapdbinst/sqllib/db2profile
       db2 connect to $ldapdbname
       db2 create "$uniq_flag index $tabschema.$indx_name on $tabschema.$tbl($indx_def)"
       db2 runstats on table $tabschema.$tbl with distribution and detailed indexes all

Note that it could take several minutes to create an index.  For example, on
an average speed machine with 4 million LDAP entries, it takes about 40 minutes
to create the objectclass(objectclass,eid) index.

EOF
					askif "Perform the recommended recovery action" y
					if [ $doit = 1 ];then
						db2 create "$uniq_flag index $tabschema.$indx_name on $tabschema.$tbl($indx_def)"
						db2 runstats on table $tabschema.$tbl with distribution and detailed indexes all
					fi
				fi
			fi
		fi
	fi
done

if [ $found = 0 ];then
	print "  No missing non-EID indexes found"
fi

rm -f potential_missing.tmp

print "Checking for additional non-EID indexes (information purposes only)"

diff nei_needed_short.tmp nei_short.tmp | grep ">" | \
	$AWK '{
		if ($2 != "EXPLAIN_ARGUMENT:+EXPLAIN_TIME+EXPLAIN_LEVEL+STMTNO+SECTNO+OPERATOR_ID:D" &&
		    $2 != "EXPLAIN_OBJECT:+EXPLAIN_TIME+EXPLAIN_LEVEL+STMTNO+SECTNO:D" &&
		    $2 != "EXPLAIN_OPERATOR:+EXPLAIN_TIME+EXPLAIN_LEVEL+STMTNO+SECTNO+OPERATOR_ID:D" &&
		    $2 != "EXPLAIN_PREDICATE:+EXPLAIN_TIME+EXPLAIN_LEVEL+STMTNO+SECTNO+OPERATOR_ID:D" &&
		    $2 != "EXPLAIN_STATEMENT:+EXPLAIN_TIME+EXPLAIN_LEVEL+STMTNO+SECTNO:D" &&
		    $2 != "EXPLAIN_STREAM:+EXPLAIN_TIME+EXPLAIN_LEVEL+STMTNO+SECTNO:D" &&
		    $2 != "SECDN:+SECDN+EID:D" &&
		    $2 != "SECDN:+RSECDN+EID:D" &&
		    $2 != "SECDN:+SECDN_T+EID:D" &&
		    $2 != "SECDN:+RSECDN_T+EID:D" \
		){
			print $2
		}
	}' >additional_idxs.tmp

found=0
for i in `cat additional_idxs.tmp`;do

	tbl=`print $i | $AWK -F ":" '{print $1}'`

	# Determine whether the table as any data

	db2 "declare c cursor with hold for select * from $tabschema.$tbl" >/dev/null
	db2 "open c" >/dev/null
	db2 fetch c for 1 row | grep selected | $AWK '{print $1}' >hasrow.tmp
	hasrow=`cat hasrow.tmp`
	rm -f hasrow.tmp
	db2 "close c" >/dev/null

	if [ $hasrow = 1 ];then

		found=1

		idx=`print $i | $AWK '{print substr($1,index($1,":")+1)}'`
		full_name=`cat noneid_idxs.tmp | grep $tbl | grep $idx`
		idx_name=`print $full_name | $AWK -F ":" '{print $2}'`
		print $full_name
		print "  If this index is not needed, run the following command to drop it."
		print "  If you are not sure, leave the index defined."
		print "       su - $ldapdbuser"
		print "       . ~$ldapdbinst/sqllib/db2profile"
		print "       db2 connect to $ldapdbname"
		print "       db2 drop index $tabschema.$idx_name"

		askif "Drop this index" n
		if [ $doit = 1 ];then
			db2 drop index $tabschema.$idx_name
		fi
	fi
done

if [ $found = 0 ];then
	print "  No additional non-EID indexes found"
fi

rm -f additional_idxs.tmp

rm -f nei_needed_short.tmp
rm -f nei_short.tmp

rm -f noneid_idxs_needed.tmp
rm -f noneid_idxs.tmp
rm -f all_tables.tmp

db2 terminate >/dev/null

