#!/bin/ksh

# Restrictions:
# - This script must be run under the context of the root system user.
# - It has only been tested on Unix platforms, primarily AIX, Solaris and Linux.
# - It does not require write authority to the current directory.

if [ `uname` = "SunOS" ];then
        AWK=nawk
else
        AWK=awk
fi

# Find the am_tune_ldap.sh script
# If it is in the current path, set command path to null
which am_tune_ldap.sh >/dev/null 2>&1
if [ $? = 0 ];then
	am_tune_ldap_cmd=am_tune_ldap.sh
else
	# not in current command path; try to get the path from the command
	cmdpath=`echo $0 | $AWK -F"/" '{for (i=1;i<NF;i++){p=p""$i"/"}}END{print p}'`
	am_tune_ldap_cmd=$cmdpath""am_tune_ldap.sh

	# verify the path to am_tune_ldap.sh
	which $am_tune_ldap_cmd >/dev/null 2>&1
	if [ $? != 0 ];then
		print "$0 Error: Cannot find am_tune_ldap.sh"
		exit 1
	fi
fi

# import the am_tune_ldap.sh script
detailed_logging=0
. $am_tune_ldap_cmd import

ask_ldapdbinst
stop_ldap	# stop the ldap server

### Database configuration parameter tuning ###

# Set a DB2 database configuration parameter, if not already set
set_db_parm_if_dif(){
	dbparm_name=$1
	recommended_value=$2

	# First, get the current setting
	cur_val=`db2 get database configuration for $ldapdbname | grep $dbparm_name | $AWK '{print $NF}'`

	print ""
	if [ "X$cur_val" = "X$recommended_value" ];then
		print "$dbparm_name is at the recommended setting of $cur_val."
		print "No change is necessary."
	else
		print "$dbparm_name is set to $cur_val, but the recommended setting is $recommended_value."
		echo db2 update database configuration for $ldapdbname using $dbparm_name $recommended_value
		db2 update database configuration for $ldapdbname using $dbparm_name $recommended_value

		dbchanged=1
	fi
}
# initialize
dbchanged=0

# Main db2 tuning

# Ensure that DB2 has been started
db2start >/dev/null

print ""
print "DB2 database configuration parameter tuning"
print ""

# Get the current db config settings
print "The current settings of some key DB2 configuration parameters are as follows:"
print ""
echo "db2 get database configuration for $ldapdbname | egrep 'DBHEAP|SORTHEAP|MAXLOCKS|MINCOMMIT|UTIL_HEAP_SZ|APPLHEAPSZ'"
db2 get database configuration for $ldapdbname | egrep 'DBHEAP|SORTHEAP|MAXLOCKS|MINCOMMIT|UTIL_HEAP_SZ|APPLHEAPSZ'

# Tune the db config settings
print ""
print "Setting the values of some key DB2 configuration parameters, if not already set."

# Need to verify these help before recommending them
#set_db_parm_if_dif SORTHEAP 2500
#set_db_parm_if_dif MAXLOCKS 100
#set_db_parm_if_dif APPLHEAPSZ 2048

# recommend setting MINCOMMIT to 1 if not already set.
#db2 update database configuration for $ldapdbname using MINCOMMIT 1
set_db_parm_if_dif MINCOMMIT 1

if [ $dbchanged = 1 ];then

	# Make sure all connections are terminated for changes to take effect
	echo db2 terminate
	db2 terminate
	echo db2 force applications all
	db2 force applications all
	sleep 2
fi

print ""
print "Press <Enter> to tune the DB2 buffer pools."
read ans

### DB2 buffer pool tuning ####

# Display the memory usage for the current set of buffer pool settings
get_buffer_pool_memory_usage(){

	db2 "select varchar(bpname,20) as bpname,npages,pagesize from syscat.bufferpools" | $AWK 'BEGIN{
		l1 = l2 = l3 = sprintf("   ")
		first = 1
	}{
		if (!past_header){
			if (index($0,"-----")){
				past_header = 1
			}
		}else if ($1=="" || skip_to_end){
			skip_to_end = 1
		}else if (first){
			first = 0
			l1 = l1 "" sprintf("%s",$1)
			l2 = l2 "" sprintf("%d*%d",$2,$3)
			l3 = l3 "" sprintf("%3.1f",$2*$3/1024/1024)
			sum += $2*$3
		}else{
			l1 = l1 "" sprintf(" + %s",$1)
			l2 = l2 "" sprintf(" + %d*%d",$2,$3)
			l3 = l3 "" sprintf(" + %3.1f",$2*$3/1024/1024)
			sum += $2*$3
		}
	}END{
		printf("%s = \n",l1)
		printf("%s = %d bytes\n",l2,sum)
		printf("%s = %3.1f MB\n",l3,sum/1024/1024)
	}'
}

# Set a DB2 buffer pool parameter, if not already set
set_bufpool_if_dif(){
	bpname=$1
	recommended_value=$2

	cur_val=`db2 connect to $ldapdbname >/dev/null 2>&1;
		db2 "select varchar(bpname,20) as bpname,npages,pagesize from syscat.bufferpools" | 
		grep -i $bpname | $AWK '{print $2}'`

	print ""
	if [ "X$cur_val" = "X$recommended_value" ];then
		print "$bpname is at the recommended size of $cur_val."
		print "No change is necessary."
	else
		print "$bpname is $cur_val, but the recommended size is $recommended_value."

		echo db2 alter bufferpool $bpname size $recommended_value
		db2 alter bufferpool $bpname size $recommended_value

		bpchanged=1
	fi
}

# initialize buffer pool changed flag
bpchanged=0

print "DB2 database buffer pool tuning"
print ""

echo db2 connect to $ldapdbname
db2 connect to $ldapdbname

# Get the current buffer pool settings
print "The current buffer pool sizes are as follows:"
print ""
echo db2 "select varchar(bpname,20) as bpname,npages,pagesize from syscat.bufferpools"
db2 "select varchar(bpname,20) as bpname,npages,pagesize from syscat.bufferpools"

print "The current buffer pool memory usage is as follows:"
print ""

get_buffer_pool_memory_usage

print ""
print "The current file system caching options are as follows:"
print ""

echo "db2 get snapshot for tablespaces on idsdb | egrep 'Tablespace name|File system caching'"
db2 get snapshot for tablespaces on idsdb | egrep 'Tablespace name|File system caching'

# Tune the buffer pool

cat <<EOF

DB2 versions 8.2 and greater support an option to turn off file system caching.  If that
option is enabled, the Operating System uses about 1 GB less memory than it would with
file system caching turned on.  Turning off file system caching will result in slower
performance unless the extra 1 GB of memory is allocated to the DB2 buffer pools.  The
following buffer pool settings are optimum for a system with 2 GB memory.

Not all file system and operating systems support the option to turn off file system
caching.  Refer to DB2 documentation for more information.

EOF

askif "Does this environment support the DB2 option to turn off file system caching" y
if [ $doit = 1 ];then

	# TAM recommendation with file system caching turned off
	# using 1 GB of memory
	# mem usage:
	#   IBMDEFAULTBP + LDAPBP = 
	#   384.0 + 640.0 = 1024.0 MB
	# ratio:  1 to 1.67
	# Turn off file system caching
	echo db2 alter tablespace USERSPACE1 no file system caching
	db2 alter tablespace USERSPACE1 no file system caching
	echo db2 alter tablespace LDAPSPACE no file system caching
	db2 alter tablespace LDAPSPACE no file system caching
	# Set the buffer pool sizes
	set_bufpool_if_dif ibmdefaultbp 98304
	set_bufpool_if_dif ldapbp 20480

else

	# TAM recommendation with file system caching turned on
	# mem usage:
	#   IBMDEFAULTBP + LDAPBP = 
	#   194.5 + 64.8 = 259.4 MB
	# ratio:  3 to 1
	set_bufpool_if_dif ibmdefaultbp 49800
	set_bufpool_if_dif ldapbp 2075

fi

# The following are some other examples of DB2 buffer pool settings

# IDS 5.1 and 6.0 defaults
# mem usage:
#   IBMDEFAULTBP + LDAPBP = 
#   115.2 + 38.4 = 153.7 MB
# ratio:  3 to 1
#set_bufpool_if_dif ibmdefaultbp 29500
#set_bufpool_if_dif ldapbp 9800

# IDS 6.0 tuning guide example configuration
# mem usage:
#   IBMDEFAULTBP + LDAPBP = 
#   78.1 + 306.3 = 384.4 MB
# ratio:  1 to 3.9
#set_bufpool_if_dif ibmdefaultbp 20000
#set_bufpool_if_dif ldapbp 9800

# Halving the TAM recommendation
# mem usage:
#   IBMDEFAULTBP + LDAPBP = 
#   97.3 + 32.4 = 129.7 MB
# ratio:  3 to 1
#set_bufpool_if_dif ibmdefaultbp 24900
#set_bufpool_if_dif ldapbp 1038

# Doubling the TAM recommendation
# mem usage:
#   IBMDEFAULTBP + LDAPBP = 
#   389.1 129.7 = 518.8 MB
# ratio:  3 to 1
#set_bufpool_if_dif ibmdefaultbp 99600
#set_bufpool_if_dif ldapbp 4150

if [ $bpchanged = 1 ];then
	echo ""
	echo db2 terminate
	db2 terminate
	echo db2 force applications all
	db2 force applications all
	sleep 2
	echo db2stop
	db2stop
	echo db2start
	db2start

	# Verify new settings

	echo db2 connect to $ldapdbname
	db2 connect to $ldapdbname
	print "The new buffer pool sizes are as follows:"
	print ""
	echo db2 "select varchar(bpname,20) as bpname,npages,pagesize from syscat.bufferpools"
	db2 "select varchar(bpname,20) as bpname,npages,pagesize from syscat.bufferpools"

	print "The new buffer pool memory usage is as follows:"
	print ""
	get_buffer_pool_memory_usage
	print ""

	print "The current file system caching options are as follows:"
	print ""
	echo "db2 get snapshot for tablespaces on idsdb | egrep 'Tablespace name|File system caching'"
	db2 get snapshot for tablespaces on idsdb | egrep 'Tablespace name|File system caching'
	print ""

	echo db2 terminate
	db2 terminate
fi

print ""
print "Press <Enter> to tune the transaction logs."
read ans

### DB2 transaction log tuning

askif "Describe the DB2 transaction log tuning" y
if [ $doit = 1 ];then

	cat << EOF

DB2 transaction log space is defined by the LOGFILSIZ, LOGPRIMARY, LOGSECOND,
NEWLOGPATH DB2 parameters. These parameters must be tuned to allow the
transaction logs to grow to their maximum required size.  The disk space in the
directory specified in the NEWLOGPATH DB2 parameters must be verified to
sufficiently large to allow the transaction logs to grow the limits defined by
the DB2 parameters.  If there is an insufficient amount of disk space, the
database becomes corrupted and goes into an unusable state.  In some cases DB2
commands can be used to recover the database and in others, the database must be
reloaded.

In normal use of the IBM Tivoli Directory Server, the transaction log
requirements are small.  Some directory operations will increase the transaction
log requirements for a short period of time.   Here are a few examples:

- The ldapadd or ldapmodify commands use an increasing amount of transaction log
  space as the number of multi-valued attributes added to a single LDAP entry in
  a single command grows, for example as in loading a group with many members.
  The transaction log requirements to load a 3 million user group is around 300
  MB.

Note: With IDS version 6.0 it is no longer necessary to use the
ldapadd/ldapmodify commands to add many members to an existing group. IDS
version 6.0 bulkload supports an option to add members to an existing group.
Using the IDS bulkload utility to add members to a group is a good way to
overcome the per user transaction log requirements.  The bulkload utility uses
very little transaction log space.

- An ACL placed on a suffix object can result in that ACL propogating to every
  entry under that suffix. The IBM directory server performs ACL propagation as
  one single committed DB2 transaction. The transaction log requirements to
  propagate an ACL to a suffix with 3 million Access Manager users in the old
  format is around 1.2 GB.  The Access Manager fixacls.sh tuning guide script
  has the same transaction log requirement as ACL propagation, since it performs
  a very similar function.

Using the 1.2 GB requirement for the propagation of ACLs to 3 million Access
Manager users, the transaction log requirements are approximately

1.2*1024 MB / 3 million users = 410 bytes per user

By default the DB2 transaction log file size  (LOGFILSIZ) is defined to be 2000
blocks of 4KB in size or 8000 KB per log file.  The number of primary logs files
(LOGPRIMARY) is defined as 3 and the number of secondary log files (LOGSECOND)
is 2.  The following formula shows the default transaction log space limit:

2000 * 4096 * ( 3 + 2 ) = 39.1 MB

The default is sufficient for approximately 100 thousand users.  In order to
increase the transaction logs DB2 limits to allow for millions of users, it is
necessary to increase the size of the transaction logs (LOGFILSIZ) and increase
the number of secondary files (LOGSECOND).  It is better to increase the number
of secondary files rather than the number of primary files, because the
secondary files periodically get deleted when not in use.

The db2_tuning.sh script can be used to adjust the DB2 transaction log
parameters for the worst case of propagating ACLs.  This script modifies the
transaction log file size (LOGFILSIZE) to 10000 blocks of 4KB in size or about
39 MB per log file.  The script modifies the number of primary log files
(LOGPRIMARY) to 2.  The script prompts for the number of Access Manager users
that will be loaded into the directory server and uses the following formula to
determine the number of secondary buffers (logsecond) to set.

LOGSECOND = (<num AM users> * 410) / <39MB log file size> - 2 primary log file + 1

EOF

print "Press <Enter> to continue database tuning."
read ans

fi

# initialize
dbchanged=0

# Get the current settings
print "The current transaction log settings are as follows:"
print ""
echo db2 get database configuration for $ldapdbname | egrep 'LOGFILSIZ|LOGPRIMARY|LOGSECOND|NEWLOGPATH|Path to log files'
db2 get database configuration for $ldapdbname | egrep 'LOGFILSIZ|LOGPRIMARY|LOGSECOND|NEWLOGPATH|Path to log files'

print ""
print "Setting the transaction log values, if not already set."

set_db_parm_if_dif LOGPRIMARY 2
set_db_parm_if_dif LOGFILSIZ 10000

while [ X"$num_tam_users" = X ];do
	num_tam_users_default=3000000
	print ""
	print "Enter the estimated number of Access Manager users that will be loaded into the Directory Server."
	print -n "This number will be used to define the transaction log size (default: $num_tam_users_default): "
	read ans

	if [ X$ans = X ];then
		ans=$num_tam_users_default
	fi

	# if it is not numeric
	if [ `awk -v ans=$ans 'BEGIN{print ans*1}'` = 0 ];then
		print "The specified number is not numeric."
	else
		num_tam_users=$ans
	fi
done

# num secondary buffers = (<num AM users> * 410) / <39MB log file size> - 2 primary log file + 1
logsecond=`awk -v num_tam_users=$num_tam_users 'BEGIN{printf("%d",(num_tam_users*410)/(39*1024*1024)-2+1)}'`
if [ $logsecond -lt 0 ];then
	logsecond=0
fi
set_db_parm_if_dif LOGSECOND $logsecond

if [ $dbchanged = 1 ];then

	# Make sure all connections are terminated for changes to take effect
	print ""
	echo db2 terminate
	db2 terminate
	echo db2 force applications all
	db2 force applications all
	sleep 2
	echo db2 connect to $ldapdbname
	db2 connect to $ldapdbname

	print ""
	print "The new transaction log settings are as follows:"
	print ""
	echo db2 get database configuration for $ldapdbname | egrep 'LOGFILSIZ|LOGPRIMARY|LOGSECOND|NEWLOGPATH|Path to log files'
	db2 get database configuration for $ldapdbname | egrep 'LOGFILSIZ|LOGPRIMARY|LOGSECOND|NEWLOGPATH|Path to log files'
fi

print ""
print "Verifying that the file system for the transaction logs is large enough to"
print "accommodate the maximum growth."
print ""

print "Filesystem limits"
echo "logfilpath=\`db2 get db cfg for $ldapdbname | grep "Path to log files" | $AWK '{print \$NF}'\`"
echo "logfilpath="`db2 get db cfg for $ldapdbname | grep "Path to log files" | $AWK '{print $NF}'`
logfilpath=`db2 get db cfg for $ldapdbname | grep "Path to log files" | $AWK '{print $NF}'`

echo "df -k $logfilpath"
df -k $logfilpath
fs_check=`df -k $logfilpath | $AWK 'BEGIN{getline}{print $0}'`

if [ `uname -s` = SunOS ];then
	echo "avail=\`echo $fs_check | $AWK '{print \$4 / 1024 }'\`"
	echo "avail="`echo $fs_check | $AWK '{print $4 / 1024 }'`
	avail=`echo $fs_check | $AWK '{print $4 / 1024 }'`
else
	echo "avail=\`echo $fs_check | $AWK '{print \$3 / 1024 }'\`"
	echo "avail="`echo $fs_check | $AWK '{print $3 / 1024 }'`
	avail=`echo $fs_check | $AWK '{print $3 / 1024 }'`
fi

print ""
print "Database log space limits"
echo "logfilsiz=\`db2 get database configuration for $ldapdbname | grep LOGFILSIZ | $AWK '{print \$NF}'\`"
echo "logfilsiz="`db2 get database configuration for $ldapdbname | grep LOGFILSIZ | $AWK '{print $NF}'`
logfilsiz=`db2 get database configuration for $ldapdbname | grep LOGFILSIZ | $AWK '{print $NF}'`

echo "logprimary=\`db2 get database configuration for $ldapdbname | grep LOGPRIMARY | $AWK '{print \$NF}'\`"
echo "logprimary="`db2 get database configuration for $ldapdbname | grep LOGPRIMARY | $AWK '{print $NF}'`
logprimary=`db2 get database configuration for $ldapdbname | grep LOGPRIMARY | $AWK '{print $NF}'`

echo "logsecond=\`db2 get database configuration for $ldapdbname | grep LOGSECOND | $AWK '{print \$NF}'\`"
echo "logsecond="`db2 get database configuration for $ldapdbname | grep LOGSECOND | $AWK '{print $NF}'`
logsecond=`db2 get database configuration for $ldapdbname | grep LOGSECOND | $AWK '{print $NF}'`

echo "logspace=( logfilsiz * 4096 * ( logprimary + logsecond ) / 1024 / 1024 )"
echo "logspace=( $logfilsiz * 4096 * ( $logprimary + $logsecond ) / 1024 / 1024 )"
echo "logspace="$(( $logfilsiz * 4096 * ( $logprimary + $logsecond ) / 1024 / 1024 ))
logspace=$(( $logfilsiz * 4096 * ( $logprimary + $logsecond ) / 1024 / 1024 ))

print ""
print "The available file space on $logfilpath is $avail MB."
print "The log parameters allow the log files to grow to a maximum of $logspace MB."
print ""

if [ $avail -lt $logspace ];then
	print "Warning: The maximum allowed log space exceeds the available log space."
	print "Consider increasing the storage in the file system or decreasing the"
	print "log space parameters.  Running out of file space for transaction logs"
	print "can corrupt the database and require it to be restored."
else
	print "Check succeeded:  There is sufficient disk space to allow for the maximum log"
	print "file growth."
fi
print ""

