#!/bin/ksh

# Script to tune DB2 table statistics such that small tables, those with statistics
# indicating less than 10,000 rows, are modified such that the statistics indicate
# 10,000 rows.  The specific DB2 table statistic that indicates the number of rows
# in a table is the CARD column of the SYSSTAT.TABLES table.

# Restrictions:
# - This script must be run under the context of the root system user.
# - It has only been tested on Unix platforms, primarily AIX, Solaris and Linux.
# - It requires write authority to the current directory.

if [ `uname` = "SunOS" ];then
        AWK=nawk
else
        AWK=awk
fi

# Find the am_tune_ldap.sh script
# If it is in the current path, set command path to null
which am_tune_ldap.sh >/dev/null 2>&1
if [ $? = 0 ];then
	am_tune_ldap_cmd=am_tune_ldap.sh
else
	# not in current command path; try to get the path from the command
	cmdpath=`echo $0 | $AWK -F"/" '{for (i=1;i<NF;i++){p=p""$i"/"}}END{print p}'`
	am_tune_ldap_cmd=$cmdpath""am_tune_ldap.sh

	# verify the path to am_tune_ldap.sh
	which $am_tune_ldap_cmd >/dev/null 2>&1
	if [ $? != 0 ];then
		print "$0 Error: Cannot find am_tune_ldap.sh"
		exit 1
	fi
fi

# import the am_tune_ldap.sh script
detailed_logging=0
. $am_tune_ldap_cmd import


ask_ldapdbinst

get_db2_level

wrong_vers=0
if [ $db2vers -lt 8 ];then
	wrong_vers=1
elif [ $db2fp -lt 8 ];then
	wrong_vers=1
fi

if [ $wrong_vers = 1 ];then
	print ""
	print "This version of DB2 does not support volatile tables."
	print ""
	exit
fi

echo ""
# Connect to the database
echo "db2 connect to $ldapdbname >/dev/null"
db2 connect to $ldapdbname >/dev/null

echo ""
print "Obtaining the number of LDAP tables that that are volatile"
echo "\tdb2 -x \"select tabname from syscat.tables where volatile = 'C' and tabschema = '$tabschema'\" | wc -l"
db2 -x "select tabname from syscat.tables where volatile = 'C' and tabschema = '$tabschema'" >$0.tmp
num_volatile_tabs=`cat $0.tmp | wc -l`
echo "$num_volatile_tabs"

if [ $num_volatile_tabs = 0 ];then

	print "There are no LDAP tables that are volatile.  No action is necessary."
	print ""
	rm $0.tmp

	# Disconnect from the database
	db2 terminate >/dev/null

	exit
fi

echo ""
echo "Setting all volatile $ldapdbname tables to not volatile"
echo ""

for i in `cat $0.tmp`;do
	echo "db2 alter table $tabschema.$i not volatile"
	db2 alter table $tabschema.$i not volatile
done

rm $0.tmp

# Disconnect from the database
echo "db2 terminate >/dev/null"
db2 terminate >/dev/null

