#!/bin/ksh

# Script invoke the SQL explain utility

# Restrictions:
# - This script must be run under the context of the root system user.
# - It has only been tested on Unix platforms, primarily AIX, Solaris and Linux.
# - It requires write authority to the current directory.

usage(){
	echo "Usage:  $0 <file containing sql statements>"
	echo "      The sql statements in the file should not be preceded with db2"
	echo "      For example, the following is a sample of what could be in the file:"
	echo "UPDATE LDAPDB2.SRC SET ACLSRC = ? WHERE ACLSRC = ? AND LDAPDB2.SRC.EID IN (SELECT DEID FROM LDAPDB2.LDAP_DESC WHERE (AEID= ?) )"
	echo ""
	exit 1
}

if [ X$1 = X ];then
	usage
fi

sqlfile=$1

if [ `uname` = "SunOS" ];then
        AWK=nawk
else
        AWK=awk
fi

# Find the am_tune_ldap.sh script
# If it is in the current path, set command path to null
which am_tune_ldap.sh >/dev/null 2>&1
if [ $? = 0 ];then
	am_tune_ldap_cmd=am_tune_ldap.sh
else
	# not in current command path; try to get the path from the command
	cmdpath=`echo $0 | $AWK -F"/" '{for (i=1;i<NF;i++){p=p""$i"/"}}END{print p}'`
	am_tune_ldap_cmd=$cmdpath""am_tune_ldap.sh

	# verify the path to am_tune_ldap.sh
	which $am_tune_ldap_cmd >/dev/null 2>&1
	if [ $? != 0 ];then
		print "$0 Error: Cannot find am_tune_ldap.sh"
		exit 1
	fi
fi

# import the am_tune_ldap.sh script
detailed_logging=0
. $am_tune_ldap_cmd import


ask_ldapdbinst

echo ""
# # Connect to the database
# echo "db2 connect to $ldapdbname >/dev/null"
# db2 connect to $ldapdbname >/dev/null

# Commands will be piped to and from this process
start_db2_ksh

write_db2_ksh "db2 list tables for all | grep EXPLAIN_ >$0.tmp"
read_db2_ksh verbose

echo mv ~$ldapdbuser/$0.tmp .
mv ~$ldapdbuser/$0.tmp .

if [ "X`cat $0.tmp`" = "X" ];then

	print Performing one time setup ...
	write_db2_ksh "db2 -tf ~$ldapdbinst/sqllib/misc/EXPLAIN.DDL"
	read_db2_ksh verbose

fi
rm $0.tmp

# echo "db2 list tables for all | $AWK '{if ($1==\"EXPLAIN_INSTANCE\"){print $2;exit}}'"
# db2 list tables for all | $AWK '{if ($1=="EXPLAIN_INSTANCE"){print $2;exit}}' >$0.tmp
# explnschem=`cat $0.tmp`
# rm $0.tmp
# print "\t$explnschem"

# # Disconnect from the database
# echo "db2 terminate >/dev/null"
# db2 terminate >/dev/null

write_db2_ksh "db2 set current explain mode explain"
read_db2_ksh verbose

if [ -f "./$sqlfile" ];then
	sqlfile=$PWD/$sqlfile
fi
sqlfile_out=$sqlfile.exfmt

write_db2_ksh "db2 -f $sqlfile"
read_db2_ksh verbose

write_db2_ksh "db2 commit work"
read_db2_ksh verbose

write_db2_ksh "db2 connect reset"
read_db2_ksh verbose

write_db2_ksh "db2exfmt -d $ldapdbname -g TIC -e $tabschema -n % -s % -w -1 -# 0 -o $0.tmp"
read_db2_ksh verbose

echo mv ~$ldapdbuser/$0.tmp $sqlfile_out
mv ~$ldapdbuser/$0.tmp $sqlfile_out

stop_db2_ksh

