#!/bin/ksh

# Script to do statement monitoring

# This script writes DB2 monitor output to the DB2 database instance owner's home
# directory and to the current directory.  The main output file is dstate.out and
# is located in the current directory.

# Restrictions:
# - This script must be run under the context of the root system user.
# - It has only been tested on Unix platforms, primarily AIX, Solaris and Linux.
# - It requires write authority to the current directory.

if [ `uname` = "SunOS" ];then
        AWK=nawk
else
        AWK=awk
fi

# Find the am_tune_ldap.sh script
# If it is in the current path, set command path to null
which am_tune_ldap.sh >/dev/null 2>&1
if [ $? = 0 ];then
	am_tune_ldap_cmd=am_tune_ldap.sh
else
	# not in current command path; try to get the path from the command
	cmdpath=`echo $0 | $AWK -F"/" '{for (i=1;i<NF;i++){p=p""$i"/"}}END{print p}'`
	am_tune_ldap_cmd=$cmdpath""am_tune_ldap.sh

	# verify the path to am_tune_ldap.sh
	which $am_tune_ldap_cmd >/dev/null 2>&1
	if [ $? != 0 ];then
		print "$0 Error: Cannot find am_tune_ldap.sh"
		exit 1
	fi
fi

# import the am_tune_ldap.sh script
detailed_logging=0
. $am_tune_ldap_cmd import


ask_ldapdbinst

echo ""
# Connect to the database
echo "db2 connect to $ldapdbname >/dev/null"
db2 connect to $ldapdbname >/dev/null

echo "db2 list tables for all | grep EXPLAIN_"
db2 list tables for all | grep EXPLAIN_ >$0.tmp
if [ "X`cat $0.tmp`" = "X" ];then

	print Performing one time setup ...
	echo db2 -tf ~$ldapdbinst/sqllib/misc/EXPLAIN.DDL
	db2 -tf ~$ldapdbinst/sqllib/misc/EXPLAIN.DDL

fi
rm $0.tmp

# Monitoring statements

monitor_dir=`echo ~$ldapdbinst`

doit=1
if [ -f dstate.out ];then

	cat << EOF

DB2 statement monitor output will be written to the file: dstate.out
This file will be overwitten.  It's current contents will be lost.

EOF
	askif "Continue with the DB2 monitor commands" y

fi
if [ $doit = 1 ];then

	echo "db2 drop event monitor dstatement >/dev/null"
	db2 drop event monitor dstatement >/dev/null

	echo "db2 \"create event monitor dstatement for statements write to file '$monitor_dir'\""
	db2 "create event monitor dstatement for statements write to file '$monitor_dir'"

	echo db2 set event monitor dstatement state 1
	db2 set event monitor dstatement state 1

	print Do experiment to be monitored now.  Press enter to stop monitoring.
	read dummyvar

	echo db2 set event monitor dstatement state 0
	db2 set event monitor dstatement state 0

	echo db2evmon -path $monitor_dir >dstate.out
	db2evmon -path $monitor_dir >dstate.out

	# Maybe use mon.awk to further process the output
	# $AWK -f proc_stmt_mon_output.awk dstate.out

	echo db2 drop event monitor dstatement
	db2 drop event monitor dstatement

	print ""
	print "DB2 statement monitor output has been written to the file: dstate.out"
	print ""

fi

# Disconnect from the database
echo "db2 terminate >/dev/null"
db2 terminate >/dev/null

