#!/bin/ksh

# Script to extract ACLs from an ldif input stream.
#
# By default, this script excludes Access Manager user ACLs.
# By adding the word "all" as a command line parameter, the script
# can be directed to extract all ACLs, including Access Manager
# ACLs.
#
# Input: standard input device containing ldif data
# Output: standard output device with extracted acl definitions

if [ `uname` = "SunOS" ];then
	AWK=nawk
else
	AWK=awk
fi

if [ X$1 = X ];then
	opt=0
else
	if [ $1 != "all" ];then
		print Usage: $0 [all]
		print "     where \"all\" indicates to extract all ACLs, e.g. not exclude Access Manager user ACLs"
		print "     by default Access Manager user ACLs are excluded"
		exit 1
	else
		opt=$1
	fi
fi

# main 

$AWK -v opt=$1 '{

	# process a line of ldif

	if ($1 == "dn:"){

		dnln_cnt = 0
		dn_line[dnln_cnt++] = $0
		notdone = 1
		while (notdone) {
			getline
			if (substr($0,1,1)==" "){
				dn_line[dnln_cnt++] = $0
			}else{
				notdone = 0
			}
		}
	}

	# If we are processing an acl
	if (in_acl){

		# if in an acl, print it out
		print $0

		# check for end of object definition
		if ($0 == ""){
			in_acl = 0
		}

	} else if ((opt == "all" || am_user_obj == 0) && 
		($1 == "aclentry:" || $1 == "aclpropagate:" || 
		$1 == "entryowner:" || $1 == "ownerpropagate:" ||
		$1 == "aclsource:" || $1 == "ownersource:")) {
	# previous acl inclusion not known, but found an acl attribute

		in_acl = 1

		# print out changetype modify statements
		for (i=0;i<dnln_cnt;i++){
			print dn_line[i]
		}
		print "changetype: modify"
		print "add: "substr($1,1,length($1)-1)
		print $0

	} else if ($1 == "objectclass:" && $2 == "secUser"){
	# not in an acl, but found an am user object

		am_user_obj = 1

	} else if ($1 == "") {
	# not in an acl, but are between ldif object definitions

		in_acl = 0
		am_user_obj = 0

	}
	# else not in an acl or between definitions, so ignore lines
}'


