#!/bin/ksh

# Warning: This script and the commands within have not been fully tested
# or documented.  Use at your own risk.

# Script to invoke the iPlanet LDAP directory db2ldif and ldif2db utilities
# to bulk load LDIF data.  These utilities the fastest way to load data
# into the iPlanet directory.

# The primary input to this script is the standard input device, which
# contains the LDIF of the data to be bulk loaded.  Other input comes from
# variables within this script.  A good reading of this script is recommended.

# Although this script allows for breaking the input LDIF into "increments"
# this functionality provides not benefit, since the processing of the
# script is to first dump the entire directory using db2ldif, add to this
# dumped ldif the new ldif to be loaded, then invoke ldif2db.  Each load
# reloads the entire database.  For this reason, set the "increment" variable
# described high enough to load all object in the input stream.

# This script breaks up the input stream of LDIF into "increments", the size
# of which are specified by the "increment" variable within this script.  The
# default setting of this variable is 2000100 which is a little over 2 million
# LDAP objects.  The reason for breaking the LDIF into increments is to
# overcome disk and memory contraints of the system on which the bulkload
# is to be performed.  Each increment loaded into LDAP with a separate
# invokation of the db2ldif/ldif2db utilities.

# Input - Standard input device containing the LDIF data to be
# bulk loaded

# Restrictions:
# This script must be run under the context of the root user.  It
# requires write authority to the current directory for temporary
# files.

# The script can be directed to stop at the next invokation of
# bulkload by writing "0" to a control file by the name of
# "continue.bulkload".
#
# The script records the dn's for first and last objects loaded in
# any increment into a log file of the scripts activities.  This
# can be used to aid restarting the script at the last known point.
#
# Disk requirements vary depending upon the LDIF input and
# the number of objects added in each incremental load.
# Disk requirements:
# - Temporary LDIF data: size of the LDIF for any given incremental load.
#   The number of objects in an incremental load is controlled by the
#   increment variable within this script.  LDIF data for a user LDAP object
#   and associated Access Manager user objects is around 900 to 1500 bytes
#   per user.  This amounts to 4 LDAP objects.  So, plan for the number of
#   bytes per increment to be around 250 to 375.  For example, for the
#   default increment of 2000100, plan for around 500 to 750 MB of storage
#   for temporary LDIF data.
# - db2ldif utility: The script first dumps the entire directory to
#   a file, the memory requirements for this file depend upon the size
#   of the directory.
# - ldif2db utility: unknown, not enough testing has been done to determine
#   how much disk, if any, this utility uses.

# Increment variable
# Note for AM users, set increment to 4 times the number of
# AM users desired per increment
increment=2000100 # a little over half a million AM users (2M LDAP objects)

export logfile=inc_bulk.out
rm -f $logfile #remove log file

# File used for temporary LDIF data
# It is not cleaned up at the end.  Do it manually.
export ldif_file=bulkload.ldif

if [ `uname` = "SunOS" ];then
	AWK=nawk
else
	AWK=awk
fi

# kill slapd if running
#slapd_pid=`ps -ef | grep ns-slapd |
#	$AWK '{if (index($0,"grep") == 0){print $2}}'`
#if [[ "X$slapd_pid" != "X" ]] then
#	print "$0: killing ns-slapd"
#	kill $slapd_pid
#fi

echo 1 >continue.bulkload

# main 

# Create a temporary ldif of the next increment
# number of ldap objects read from the input
# device and invoke the bulkload utility

rm -f $ldif_file.*

$AWK -v ldif_file=$ldif_file \
	-v index_option=$1 \
	-v increment=$increment '

function continue_check(){

	getline < "continue.bulkload"
	if ($1 != 1){

		exit
	}
	close ("continue.bulkload")
}

function invoke_bulkload(){

	continue_check()

	# stop slapd
	system(stop_cmd)
	system("sleep 1")

	# Invoke the bulkload utility

	system("echo start time ldif2db: `date` 2>&1 | tee -a $logfile")

	for (suf in all_suffixes){

		# Note the LDR_CNTRL environment variable used below increases the process size limit
		# to 2 GB on AIX systems.  It has no affect on other operating systems.

		cmd = "LDR_CNTRL=MAXDATA=0x80000000 "ldif2db_cmd" -s "suf" -i $PWD/"ldif_file"."all_suffixes[suf]
		system("echo "cmd" | tee -a $logfile")
		system(cmd" 2>&1 | tee -a $logfile")

		all_suffixes[suf] = ""
	}
	suf_num = 0

	continue_check()

	# start slapd
	system(start_cmd)
	system("sleep 1")

	system("rm -f "ldif_file".*")
}

BEGIN{
	system("echo start time create temporary ldifs: `date` 2>&1 | tee -a $logfile")

	# find the server directory.  Assume there is only one and it starts with slapd-
	"ls -d /usr/iplanet/servers/slapd-*" | getline
	serverdir = $0
	close("ls -d /usr/iplanet/servers/slapd-*")

	ldif2db_cmd = serverdir"/ldif2db"
	db2ldif_cmd = serverdir"/db2ldif"
	start_cmd = serverdir"/start-slapd"
	stop_cmd = serverdir"/stop-slapd"

}
{
	# if this is a dn line
	if ($1 == "dn:"){

		dn_line = $0

		ne=split(dn_line,dn_array,",")
		ne=split(dn_array[ne],suffix_array)
		suffix = suffix_array[ne]
		if (all_suffixes[suffix] == ""){
			all_suffixes[suffix] = ++suf_num

			# initialize the suffix ldif file with the contents of the suffix db

			cmd = "LDR_CNTRL=MAXDATA=0x80000000 "db2ldif_cmd" -s "suffix" -a $PWD/"ldif_file"."suf_num
			system("echo "cmd" | tee -a $logfile")
			system(cmd" 2>&1 | tee -a $logfile")

		}

		# save the first dn
		if (!not_first){
			not_first = 1
			system("echo first "dn_line" | tee -a $logfile")
		}

		# check for and save the last dn
		dn_count++
		if (dn_count >= increment){
			last_dn = 1
			system("echo last "dn_line" | tee -a $logfile")
		}
	}

	# put the line in the temp ldif file
	fn = ldif_file"."all_suffixes[suffix]
	print $0 >> fn

	# if working on the last dn, find the null line that
	# terminates it and invoke bulkload
	if (last_dn && $0 == ""){
		not_first = 0
		last_dn = 0
		dn_count = 0

		for (suf in all_suffixes){
			close(ldif_file"."all_suffixes[suf])
		}
		invoke_bulkload()

		system("echo start time create temporary ldif: `date` 2>&1 | tee -a $logfile")
	}
}END{

	# bulkload any left over dns
	if (not_first){
		system("echo last "dn_line" | tee -a $logfile")
		for (suf in all_suffixes){
			close(ldif_file"."all_suffixes[suf])
		}
		invoke_bulkload()
	}

}'

echo stop time: `date` 2>&1 | tee -a $logfile

