#!/bin/ksh

# greps an ldif data stream coming from the standard input device
# for ldap objects that DO NOT contain one of the specified strings from
# the parameter line.

# This script properly handles LDIF continuation lines.

usage(){
	echo "Usage:"
	echo "    $0 [-i] <string1> [<string2> ... <stringn>]"
	echo "        where \"-i\" is the ignore case option"
	echo "        <stringx> is one of a set of strings to search for in any part of an LDAP object"
	echo "        Exclude objects that contain one of these strings."
	exit 1
}


if [ "X$1" = "X-i" ];then
	ignorecase=1
	shift
else
	ignorecase=0
fi

if [ "X$1" = X ];then
	usage
fi

unset strings
while [ "X$1" != "X" ];do
	strings=$strings$1"\n"
	shift
done

awk -v ignorecase=$ignorecase -v strings="$strings" 'BEGIN{

	num_strings = split(strings,str_ary,"\n")
	num_strings -= 1

	if (ignorecase){
		for (i=1;i<=num_strings;i++){
			str_ary[i] = toupper(str_ary[i])
		}
	}
}{
	# print or save the line based on whether found
	if (found){
		# exclude it
	}else{
		# save it
		s[c++]=$0
	}

	# if blank line, initialize found flag
	if ($1==""){
		# dump the ldif object if no matching string has been found
		if (!found){
			for (i=0;i<c;i++){
				print s[i]
			}
		}

		# initialize
		found=0
		c=0
	}

	# Check for existence of input string
	if (!found){

		# if this is a continuation line, combine it
		if (substr($0,1,1)==" "){
			ldif_line = ldif_line substr($0,index($0,$1))
		}else{
			ldif_line = $0
		}

		# handle ignore case
		if (ignorecase){
			ldif_line = toupper(ldif_line)
		}

		# check if any of the input strings are found
		for (i=1;i<=num_strings&&!found;i++){
			if (index(ldif_line,str_ary[i])){
				found=1
			}
		}
	}
	# print "DEBUG: "found " " ldif_line
}END{

	# dump the ldif object if no matching string has been found
	if (!found){
		for (i=0;i<c;i++){
			print s[i]
		}
	}

}'

