#!/bin/ksh

# Sript to create the ldif for a minimum, unconfigured Access Manager subdomain

# Restrictions:
# This script can be run under the context of any user.  It does not
# require write authority to the current directory.

usage(){
	print "Usage: $0 <domain-name> <LDAP data format>"
	print "where"
	print "     <domain-name> is the name of the domain to be created"
	print "     <LDAP data format> is either \"new\" or \"old\""
	print "      Refer to the Configuration Guide for more information on LDAP data format"
	exit 1
}

if [ "X$2" != "Xnew" ]&&[ "X$2" != "Xold" ];then
	usage
fi
dom=$1
data_format=$2

# map the LDAP data format to the TAM version number
if [ $data_format = new ];then
	data_format_vers=6.0
else
	data_format_vers=3.0
fi

# Access Manager domain LDIF definition:

cat <<EOF
dn: secAuthority=$dom,cn=Subdomains,secAuthority=Default
objectclass: secAuthorityInfo
objectclass: eApplicationSystem
objectclass: eSystem
objectclass: cimLogicalElement
objectclass: cimManagedSystemElement
objectclass: cimManagedElement
objectclass: top
secAuthority: $dom
version: $data_format_vers
aclpropagate: TRUE
ownerpropagate: TRUE
entryowner: group:CN=SECURITYGROUP,SECAUTHORITY=$dom,CN=SUBDOMAINS,SECAUTHORIT
 Y=DEFAULT
aclentry: group:CN=REMOTE-ACL-USERS,CN=SECURITYGROUPS,SECAUTHORITY=$dom,CN=SUB
 DOMAINS,SECAUTHORITY=DEFAULT:at.secPwdFailCountTime:rwsc:at.secAcctValid:rws
 c:system:rsc:normal:rsc:at.secPwdValid:rwsc:at.secPwdUnlockTime:rwsc:at.secP
 wdLastUsed:rwsc:at.secPwdLastFailed:rwsc:at.secPwdLastChanged:rwsc:at.secPwd
 Failures:rwsc
aclentry: group:CN=SECURITYGROUP,SECAUTHORITY=DEFAULT:system:rsc:critical:rwsc
 :sensitive:rwsc:normal:rwsc:object:ad
aclentry: group:CN=IVACLD-SERVERS,CN=SECURITYGROUPS,SECAUTHORITY=DEFAULT:at.se
 cPwdValid:rwsc:at.secPwdUnlockTime:rwsc:at.secPwdLastUsed:rwsc:at.secPwdLast
 Failed:rwsc:at.secPwdLastChanged:rwsc:at.secPwdFailures:rwsc:at.secPwdFailCo
 untTime:rwsc:at.secAcctValid:rwsc:system:rsc:normal:rsc
aclentry: group:CN=REMOTE-ACL-USERS,CN=SECURITYGROUPS,SECAUTHORITY=DEFAULT:at.
 secPwdValid:rwsc:at.secPwdUnlockTime:rwsc:at.secPwdLastUsed:rwsc:at.secPwdLa
 stFailed:rwsc:at.secPwdLastChanged:rwsc:at.secPwdFailures:rwsc:at.secPwdFail
 CountTime:rwsc:at.secAcctValid:rwsc:system:rsc:normal:rsc
aclentry: group:CN=SECURITYGROUP,SECAUTHORITY=$dom,CN=SUBDOMAINS,SECAUTHORITY=
 DEFAULT:system:rsc:critical:rwsc:sensitive:rwsc:normal:rwsc:object:ad
aclentry: group:CN=IVACLD-SERVERS,CN=SECURITYGROUPS,SECAUTHORITY=$dom,CN=SUBDO
 MAINS,SECAUTHORITY=DEFAULT:at.secPwdValid:rwsc:at.secPwdUnlockTime:rwsc:at.s
 ecPwdLastUsed:rwsc:at.secPwdLastFailed:rwsc:at.secPwdLastChanged:rwsc:at.sec
 PwdFailures:rwsc:at.secPwdFailCountTime:rwsc:at.secAcctValid:rwsc:system:rsc
 :normal:rsc

dn: cn=Users,secAuthority=$dom,cn=Subdomains,secAuthority=Default
objectclass: container
objectclass: top
cn: Users

dn: cn=Groups,secAuthority=$dom,cn=Subdomains,secAuthority=Default
objectclass: container
objectclass: top
cn: Groups

dn: cn=SecurityGroups,secAuthority=$dom,cn=Subdomains,secAuthority=Default
objectclass: container
objectclass: top
cn: SecurityGroups

dn: cn=Policies,secAuthority=$dom,cn=Subdomains,secAuthority=Default
objectclass: container
objectclass: top
cn: Policies

dn: cn=SecurityDaemons,secAuthority=$dom,cn=Subdomains,secAuthority=Default
objectclass: container
objectclass: top
cn: SecurityDaemons

dn: cn=Subdomains,secAuthority=$dom,cn=Subdomains,secAuthority=Default
objectclass: container
objectclass: top
cn: Subdomains

dn: cn=Resources,secAuthority=$dom,cn=Subdomains,secAuthority=Default
objectclass: container
objectclass: top
cn: Resources

dn: cn=ResourceGroups,secAuthority=$dom,cn=Subdomains,secAuthority=Default
objectclass: container
objectclass: top
cn: ResourceGroups

dn: cn=SecurityGroup,secAuthority=$dom,cn=Subdomains,secAuthority=Default
objectclass: top
objectclass: groupOfNames
cn: SecurityGroup
description: DO NOT MODIFY: Access Manager identity
member: secAuthority=$dom,cn=Subdomains,secAuthority=Default

dn: cn=ivacld-servers,cn=SecurityGroups,secAuthority=$dom,cn=Subdomains,secAut
 hority=Default
objectclass: top
objectclass: groupOfNames
cn: ivacld-servers
description: DO NOT MODIFY: Controls local cache mode ACL servers
member: secAuthority=$dom,cn=Subdomains,secAuthority=Default

dn: cn=remote-acl-users,cn=SecurityGroups,secAuthority=$dom,cn=Subdomains,secA
 uthority=Default
objectclass: top
objectclass: groupOfNames
cn: remote-acl-users
description: DO NOT MODIFY: Controls remote cache mode ACL servers
member: secAuthority=$dom,cn=Subdomains,secAuthority=Default

EOF
