#!/bin/ksh

# Script to create the ldif for an LDAP group with
# many users
#
# The users to add come from the standard input device, which
# is typically piped from the "cat" of a file containing input.
#
# Input is in LDIF format.  Any DN in the ldif that has an
# objectclass of inetOrgPerson is added as a member of the group.

# The DN of the group to create one of the arguments to the script.

# Restrictions:
# This script can be run under the context of any user.  It does not
# require write authority to the current directory.

usage(){
	print ""
	print "Usage: $0 create|add <dn of group>"
	print "        where <dn of group> must begin with \"cn=\""
	print ""
	exit 1
}

if [ X$1 = X ] || [ X$2 = X ] ;then
	usage
fi

cmd=$1
groupdn=$2

if [[ $cmd != "create" ]] && [[ $cmd != "add" ]];then
	usage
fi

if [ `uname` = "SunOS" ];then
	AWK=nawk
else
	AWK=awk
fi

groupcn=`print $groupdn | $AWK '{
	if (substr($0,1,3) == "cn="){
		if (index ($0,",")){
			print substr($0,4,index($0,",")-4)
		} else {
			print $0
		}
	}
}'`

if [ X$groupcn = X ];then
	usage
fi

$AWK -v cmd=$cmd \
    -v groupdn=$groupdn \
    -v groupcn=$groupcn \
'BEGIN{

	if (cmd == "create"){

		print "dn: "groupdn
		#print "objectclass: accessGroup"
		print "objectclass: groupOfNames"
		print "objectclass: top"
		print "cn: "groupcn

	} else {

		print "dn: "groupdn
		print "changetype: modify"
		print "add: member"

	}
}
{
	if ($1 == "dn:"){
		dn = substr($0,index($0,$2))
		notdone = 1
		while (notdone) {
			getline
			if (substr($0,1,1)==" "){
				dn = dn""substr($0,index($0,$1))
			}else{
				notdone = 0
			}
		}
	}
	if (($1 == "objectClass:" || $1 == "objectclass:") && \
		($2 == "inetOrgPerson" || $2 == "inetorgperson")){

		print "member: "dn
	}
}'
