#!/bin/ksh

# Script to create the ldif for a test LDAP group with many users.
#
# The DN of the group to create is on of the argument as well as
# a user prefix

# Restrictions:
# This script can be run under the context of any user.  It does not
# require write authority to the current directory.

usage(){
	print "Usage: $0 create|add <start user number> <end user number> <groupdn> <user prefix dn>"
	print "    where"
	print "        <groupdn> is the dn of the group to create"
	print "               and must begin with \"cn=\""
	print "        <user prefix dn> is the user prefix used to create the users to add to"
	print "               the group.  For each number in the input range, a user with that"
	print "               number is added as a member of the group.  For example, if the"
	print "               input user is cn=testuser,o=ibm,c=us, an example user added to the"
	print "               group might be cn=testuser10,o=ibm,c=us."
	exit 1
}

if [ X$1 = X ]||[ X$2 = X ]||[ X$3 = X ]||[ X$4 = X ]||[ X$5 = X ];then
	usage
fi

cmd=$1
start_user=$2
end_user=$3
groupdn=$4
userdn=$5

if [[ $cmd != "create" ]] && [[ $cmd != "add" ]];then
	usage
fi

if [ `uname` = "SunOS" ];then
	AWK=nawk
else
	AWK=awk
fi

groupcn=`print $groupdn | $AWK '{
	if (substr($0,1,3) == "cn="){
		if (index ($0,",")){
			print substr($0,4,index($0,",")-4)
		} else {
			print $0
		}
	}
}'`

if [ X$groupcn = X ];then
	usage
fi

print $userdn | grep "," >/dev/null
if [ $? = 1 ];then
	print "Error: User prefix DN cannot be an LDAP suffix."
	print ""
	usage
fi

$AWK -v cmd=$cmd \
    -v start_user=$start_user \
    -v end_user=$end_user \
    -v groupdn=$groupdn \
    -v groupcn=$groupcn \
    -v userdn=$userdn \
'BEGIN{

	userprefix = substr(userdn,1,index(userdn,",")-1)
	userparent = substr(userdn,index(userdn,",")+1)

	if (cmd == "create"){

		print "dn: "groupdn
		#print "objectclass: accessGroup"
		print "objectclass: groupOfNames"
		print "objectclass: top"
		print "cn: "groupcn

	} else {

		print "dn: "groupdn
		print "changetype: modify"
		print "add: member"

	}

	for (i=start_user; i<=end_user; i++){

		print "member: "userprefix""i","userparent
	}
}'
