#!/bin/ksh

# Sript to create the ldif for test users

# Restrictions:
# This script can be run under the context of any user.  It does not
# require write authority to the current directory.

# Note that the "sn" attribute should be customized in the LDIF
# generation section of this script.

if [ `uname` = "SunOS" ];then
	AWK=nawk
else
	AWK=awk
fi

usage(){
	print "Usage: $0 <start user number> <end user number> <user prefix dn> <password>"
	print "    where"
	print "        <user prefix dn> is the user prefix used to create the users.  For each"
	print "               number in the input range, LDIF for an LDAP user with that number"
	print "               is generated and written to standard output.  For example, if the"
	print "               input user is cn=testuser,o=ibm,c=us, an example user DN might"
	print "               cn=testuser10,o=ibm,c=us."
	print "               The user prefix dn must begin with \"cn=\"."
	print "        <password> is the password that all users with be initialized to."
	print "               Note:"
	print "               Setting all passwords to the same value is not a good security"
	print "               policy.  For Access Manager, the user can be forced into a"
	print "               password change by setting the secpwdvalid attribute to false."
	print "               There is no equivalent for the LDAP."
}

if [ X$1 = X ]||[ X$2 = X ]||[ X$3 = X ]||[ X$4 = X ];then
	usage
	exit 1
fi

start_user=$1
end_user=$2
userdn=$3
passwd=$4

usercn=`print $userdn | $AWK '{
	if (substr($0,1,3) == "cn="){
		if (index ($0,",")){
			print substr($0,4,index($0,",")-4)
		} else {
			print $0
		}
	}
}'`

if [ X$usercn = X ];then
	usage
	print ""
	print "Error:  User prefix dn must begin with \"cn=\""
	print ""
	exit 1
fi

userparent=`print $userdn | $AWK '{
	print substr($0,index($0,",")+1)
}'`

# iterate through user numbers generating ldif output
num_users=$(( $end_user - $start_user + 1 ))
user_num=$start_user

# create a function to print the LDIF
print_user(){

#The LDAP user ldif follows:

	print dn: cn=$usercn$user_num,$userparent
	print objectclass: inetOrgPerson
	print objectclass: ePerson
	print objectclass: organizationalPerson
	print objectclass: person
	print objectclass: top
	print cn: $usercn$user_num
	print sn: test
	print userpassword: $passwd
	print uid: $usercn$user_num
	print 
}

while [[ $user_num -le $end_user ]] do

	print_user

	user_num=$(( $user_num + 1 ))
done

