#!/bin/ksh

usage(){
	print "Usage: $0 date|sec|both  nano|micro|milli|sec|min|hour  [echo] [<precision>]"
	print "  Where time stamped input comes from the standard input device"
	print "  - First parameter indicates what type of time stamps to process"
	print "  - Second parameter indicates in what units to output time stamps"
	print "  - \"echo\" optional parameter indicates whether to echo all lines (default is no)"
	print "  - <precision> optional parameter indicates the number of decimal places on output (default is 3)"
	exit 1
}

if [ X$1 = X ];then
	print ""
	print [$0]: At least two parameters must be specified.
	print ""
	usage
	
else
	if [ $1 != date ]&&[ $1 != sec ]&&[ $1 != both ];then
		print ""
		print [$0]: The first parameters is incorrect.
		print ""
		usage
	fi
	type=$1
fi

if [ X$2 = X ];then
	print ""
	print [$0]: The second parameter must be specified.
	print ""
	usage
else
	if [ $2 != nano ]&&[ $2 != micro ]&&[ $2 != milli ]&&[ $2 != sec ]&&[ $2 != min ]&&[ $2 != hour ];then
		print ""
		print [$0]: The second parameter is incorrect.
		print ""
		usage
	fi
	unit=$2
fi

echoall=0
precision=3

optional_parm_check(){

	if [ $1 = echo ];then
		echoall=1
	elif [ $1 = `awk -v x=$1 'BEGIN{print 1*x}'` ];then
		precision=$1
	else
		print ""
		print [$0]: An incorrect optional parameter was specified.
		print ""
		usage
	fi
}

if [ X$3 != X ];then
	optional_parm_check $3
	shift
fi
if [ X$3 != X ];then
	optional_parm_check $3
	shift
fi
if [ X$3 != X ];then
	print ""
	print [$0]: Too many parameters were specified.
	print ""
	usage
fi

awk -v type=$type -v unit=$unit -v echoall=$echoall -v precision=$precision '

function tsout(){

	if (ts-pts < 0){
		days += 1
		ts += 24*60*60
	}


	if (unit == "nano"){
		print $0" \t# " sprintf("%d",ts*1000000000) " total nano-secs " \
			sprintf("%d",(ts-pts)*1000000000) " delta nano-secs"
	}
	if (unit == "micro"){
		print $0" \t# " sprintf(precision_format,ts*1000000) " total micro-secs " \
			sprintf(precision_format,(ts-pts)*1000000) " delta micro-secs"
	}
	if (unit == "milli"){
		print $0" \t# " sprintf(precision_format,ts*1000) " total milli-secs " \
			sprintf(precision_format,(ts-pts)*1000) " delta milli-secs"
	}
	if (unit == "sec"){
		print $0" \t# " sprintf(precision_format,ts) " total secs " \
			sprintf(precision_format,ts-pts) " delta secs "
	}
	if (unit == "min"){
		print $0" \t# " sprintf(precision_format,ts/60) " total mins " \
			sprintf(precision_format,(ts-pts)/60) " delta mins "
	}
	if (unit == "hour"){
		print $0" \t# " sprintf(precision_format,ts/60/60) " total hours " sprintf(precision_format,(ts-pts)/60/60) " delta hours "
	}
}

BEGIN{
	precision_format = "%1."precision"f"
}{
	# print "DEBUG: "$0
	timestamp_found = 0
	p = match($0,/[0-9]{2}:[0-9]{2}:[0-9]{2}/)
	c_prev = substr($0,p-1,1)
	c_next = substr($0,p+8,1)
	# print "DEBUG: c_next: "c_next
	if (c_prev == ":" || (c_next != "." && c_next != " ")){
		p = 0
	}
	if ((type == "date" || type == "both") && p){
		split(substr($0,p,8),a,":")
		if (c_next == "."){
			nanosec = substr($0,p+9)
			nanosec_end_p = match(nanosec,/[^0-9]/)
			if (nanosec_end_p != 0){
				nanosec = substr(nanosec,1,nanosec_end_p-1)
			}
			nanosec = "0."nanosec
		}else{
			nanosec = 0
			if (unit != "min" && unit != "hour"){
				precision_format = "%d"
			}
		}
		ts = days*24*60*60 + a[1]*60*60 + a[2]*60 + a[3] + nanosec

		# print "DEBUG: days: "days" hours: "a[1]" mins: "a[2]" secs: "a[3]" nanosecs: "nanosec #DEBUG
		# print "DEBUG: ts: "ts #DEBUG

		if (!notfirst){
			notfirst = 1
			start_ts = ts
		}
		ts = ts - start_ts
		# print "DEBUG: ts: "ts #DEBUG
		tsout()
		timestamp_found = 1
	}

	p = index($0,"second")
	if ((type == "sec" || type == "both") && p){

		l = split(substr($0,1,p-1),a)
		ts = a[l]

		# split(substr($0,p),a)
		# if (!notfirst){
		# 	notfirst = 1
		# 	start_ts = 0
		# }
			
		# ts = pts + a[2]

		tsout()
		timestamp_found = 1
	}

	if (!timestamp_found && echoall){
		print $0
	}
	pts = ts

}'

