#!/bin/ksh

# Script to tune DB2 table statistics such that small tables, those with statistics
# indicating less than 10,000 rows, are modified such that the statistics indicate
# 10,000 rows.  The specific DB2 table statistic that indicates the number of rows
# in a table is the CARD column of the SYSSTAT.TABLES table.

# Restrictions:
# - This script must be run under the context of the root system user.
# - It has only been tested on Unix platforms, primarily AIX, Solaris and Linux.
# - It does not require write authority to the current directory.

if [ `uname` = "SunOS" ];then
        AWK=nawk
else
        AWK=awk
fi

# Find the am_tune_ldap.sh script
# If it is in the current path, set command path to null
which am_tune_ldap.sh >/dev/null 2>&1
if [ $? = 0 ];then
	am_tune_ldap_cmd=am_tune_ldap.sh
else
	# not in current command path; try to get the path from the command
	cmdpath=`echo $0 | $AWK -F"/" '{for (i=1;i<NF;i++){p=p""$i"/"}}END{print p}'`
	am_tune_ldap_cmd=$cmdpath""am_tune_ldap.sh

	# verify the path to am_tune_ldap.sh
	which $am_tune_ldap_cmd >/dev/null 2>&1
	if [ $? != 0 ];then
		print "$0 Error: Cannot find am_tune_ldap.sh"
		exit 1
	fi
fi

# import the am_tune_ldap.sh script
detailed_logging=0
. $am_tune_ldap_cmd import


ask_ldapdbinst

start_db2_ksh

echo "Obtaining the number of IDS LDAP database tables that have a number of rows (CARD column statistic) of less than 10000."

write_db2_ksh "db2 -x \"select count(*) from sysstat.tables where card < 10000 and tabschema = '$tabschema'\""
read_db2_ksh
numtabs=$db2_ksh_result
print "\t"$numtabs

if [ $numtabs = 0 ];then
	echo ""
	echo "There are no tables with less than 10,000 rows."
	echo "No action taken."
	echo ""
else
	write_db2_ksh "
	db2 \"update sysstat.tables \\
		set card=10000, \\
		npages=10000, \\
		fpages=10000, \\
		overflow=0, \\
		active_blocks=0 \\
		where card < 10000 and tabschema = '$tabschema'\""
	read_db2_ksh verbose

	# neither of these helped
	if [ 0 = 1 ];then

		write_db2_ksh "
	db2 \"update sysstat.columns \\
		set colcard=10000 \\
		where colcard < 10000 and colname <> 'EID' and \\
		tabname = 'MEMBER' and \\
		tabschema = '$tabschema'\""

		read_db2_ksh verbose

		write_db2_ksh "
	db2 -v \"update sysstat.columns \\
		set colcard=10 \\
		where colcard = -1 and colname = 'EID' and \\
		tabname = 'MEMBER' and tabschema = '$tabschema'\""

		read_db2_ksh verbose
	fi

fi

stop_db2_ksh

