#!/bin/ksh

# Script to perform a DB2 runstats on all IDS LDAP directory tables.
# Preferrably, this script uses the IDS runstats utility

# Restrictions:
# - This script must be run under the context of the root system user.
# - It has only been tested on Unix platforms, primarily AIX, Solaris and Linux.
# - It does not require write authority to the current directory.

if [ `uname` = "SunOS" ];then
        AWK=nawk
else
        AWK=awk
fi

# Find the am_tune_ldap.sh script
# If it is in the current path, set command path to null
which am_tune_ldap.sh >/dev/null 2>&1
if [ $? = 0 ];then
	am_tune_ldap_cmd=am_tune_ldap.sh
else
	# not in current command path; try to get the path from the command
	cmdpath=`echo $0 | $AWK -F"/" '{for (i=1;i<NF;i++){p=p""$i"/"}}END{print p}'`
	am_tune_ldap_cmd=$cmdpath""am_tune_ldap.sh

	# verify the path to am_tune_ldap.sh
	which $am_tune_ldap_cmd >/dev/null 2>&1
	if [ $? != 0 ];then
		print "$0 Error: Cannot find am_tune_ldap.sh"
		exit 1
	fi
fi

# import the am_tune_ldap.sh script
detailed_logging=0
. $am_tune_ldap_cmd import

# function get_stat to get the DB2 table statistics

# args:  sysstat table name, IDS table name, stat column to modify
get_stat(){

	stat_tbl=$1
	tbl=$2
	istat=$3

if [ 0 = 1 ];then

	# OLD WAY

	# For some reason, the sysstat.index table varies with user context
	# and the systat tables only show up when switched to the user in certain
	# versions of DB2 (e.g. 8.1.2),
	# so to get correct view, we just switch to the database manager user

	cat << EOFEOF
	cat << EOF | su - $ldapdbuser -c ksh
		. ~$ldapdbinst/sqllib/db2profile
		db2 connect to $ldapdbname >/dev/null
		db2 -x "select $istat from sysstat.$stat_tbl where tabname = '$tbl'"
		db2 terminate >/dev/null
EOF
EOFEOF
	cat << EOF | su - $ldapdbuser -c ksh |&
		. ~$ldapdbinst/sqllib/db2profile
		db2 connect to $ldapdbname >/dev/null
		db2 -x "select $istat from sysstat.$stat_tbl where tabname = '$tbl'"
		db2 terminate >/dev/null
EOF

	wait $!
	# END OF OLD WAY
fi
	start_db2_ksh

	write_db2_ksh "db2 -x \"select max($istat) from sysstat.$stat_tbl where tabname = '$tbl'\""
	read_db2_ksh
	stat=$db2_ksh_result

	if [ "X$stat" = "X-" ];then
		unset stat
		print "\tTable does not exist at this time."
	else
		print "\t$stat"
	fi
}

runstats_on_one_table(){

	# input arg: table name

	echo db2 runstats on table $tabschema.$1 with distribution and detailed indexes all $ACCESS
	db2 runstats on table $tabschema.$1 with distribution and detailed indexes all $ACCESS
}

# function to do the runstats using DB2
runstats_using_db2(){

	ask_ldapdbinst

	# get the DB2 level
	get_db2_level
	if [ $db2vers -ge 8 ];then
		ACCESS="shrlevel change"
	else
		ACCESS="allow write access"
	fi

	# Connect to the database
	echo db2 connect to $ldapdbname
	db2 connect to $ldapdbname >/dev/null

	# Execute runstats on all tables
	echo "db2 list tables for all | grep $tabschema"
	for i in `db2 connect to $ldapdbname > /dev/null; db2 list tables for all | grep $tabschema | cut -d' ' -f1`;do

		if [ $table_size_limit = 0 ];then

			# indicates to perform runstats on all tables
			echo db2 runstats on table $tabschema.$i with distribution and detailed indexes all $ACCESS
			db2 runstats on table $tabschema.$i with distribution and detailed indexes all $ACCESS

		else
			# limit runstats to tables of size less than table_size_limit rows

			# Get the current number of rows according to DB2 statistics
			get_stat tables $i card

			# First, does the table exist
			if [ "X$stat" != X ];then

				# Second, is it less than the limit
				if [ $stat -lt $table_size_limit ];then

					# do runstats
					runstats_on_one_table $i

				else
					# it is greater, than the limit, but have its statistics been tuned
					if [ $stat -ge 90000000000 ];then
						# Get the current number of rows according to the index statistics
						get_stat columns $tbl colcard

						# Second, is it less than the limit
						if [ $stat -lt $table_size_limit ];then
							
							# do runstats
							runstats_on_one_table $i
						fi
					fi
				fi
			fi
		fi
	done
}

# Try to use IDS runstats if the version supports it.  Othwerwise, falls back on DB2
runstats_try_ids(){

	stop_ldap

	# The directory server is not running, so use the IDS
	# runstats utilities if the version supports it.

	if [ $ldapvers -ge 600 ];then
		echo idsrunstats -I $idsldap_inst
		idsrunstats -I $idsldap_inst

	elif [ $ldapvers -ge 500 ];then
		echo runstats -f $ibmslapd_conf
		runstats -f $ibmslapd_conf

	else
		# use DB2 runstats for IDS version 4.1
		runstats_using_db2
	fi
}

# Main

cat << EOF

Performing a DB2 runstats on all tables of the directory server takes a long time
if the server has many users.  For example, with a 3 million user directory, a
DB2 runstats on a 400 MHz system takes about 20 minutes to perform on all tables.

If DB2 runstats are limited to tables of less than 100,000 rows, DB2 runstats
takes much less time, typically less than a minute.

In most cases, DB2 runstats can be limited to smaller tables.  If there is a
question about performance, perform a full runstats on all tables.

EOF

askif "Limit the DB2 runstats to tables of less than 100,000 rows" y
table_size_limit=0
if [ $doit = 1 ];then

	table_size_limit=100000
fi

# First determine whether the directory server is running

is_ldap_running # this is an am_tune_ldap.sh imported function
if [ $? = 0 ];then
	# the directory server is running

	cat << EOF

The directory server is currently running.  DB2 runstats will degrade the server
performance while it is running.  This could result in time outs and, in rare
cases with IDS versions less than 6.0, could result in deadlocks.  The deadlock
condition is a result of a potential conflict between automatic statistic
updates and the statistic updates done by runstats.  This can be avoided
by stopping the directory server or by starting the directory server with the
MAXCARD=NO environment variable.

EOF
	askif "Perform the DB2 runstats while the server is running" y
	if [ $doit = 1 ];then

		# use DB2 runstats if IDS is running
		runstats_using_db2
	else
		askif "Stop the directory server before performing the DB2 runstats" y
		if [ $doit = 1 ];then
			# Decided not to use IDS runstats.  The main difference is that IDS runstats
			# cannot be run while the directory server is running and doesn't perform all
			# the system statistic tunes (sysstat_tune.sh) that are recommended for TAM,
			# and it gives less feedback.
			# Also, IDS runstats does not have the table limit option.
			# This routine performs the IDS runtstats:  runstats_try_ids

			runstats_using_db2
		else
			print "DB2 runstats will not be performed."
			exit 1
		fi
	fi
else
	# Decided not to use IDS runstats.  The main difference is that IDS runstats
	# cannot be run while the directory server is running and doesn't perform all
	# the system statistic tunes (sysstat_tune.sh) that are recommended for TAM,
	# and it gives less feedback.
	# Also, IDS runstats does not have the table limit option.
	# This routine performs the IDS runtstats:  runstats_try_ids

	runstats_using_db2
fi

stop_db2_ksh

# Set table to volatile or small table tune

# After a runstats, it is a good idea to make sure any non-volatile tables are set to
# volatile, so that DB2 will use any index that is defined on it regardless of what it
# thinks might be faster.  The benefit is that if any small table suddenly grows and
# the index becomes important for performance, DB2 will use it, even though it thinks
# the table is small and the index has no value.
#
# For DB2 levels that do not support volatile, there is a system static tune that
# will make DB2 think tables are large enough to make the use of indexes beneficial.

get_db2_level
supports_volatile=1
if [ $db2vers -lt 8 ];then
	supports_volatile=0
elif [ $db2fp -lt 8 ];then
	supports_volatile=0
fi

if [ $supports_volatile = 1 ];then
	echo db2_volatile_tune.sh
	db2_volatile_tune.sh
else
	echo sysstat_tune_small_tables.sh
	sysstat_tune_small_tables.sh
fi

# Do the sysstat tune

# After a DB2 runstats it is necessary to re-run the sysstat_tune.sh script, since
# runstats updates the table statistics that are tuned with the sysstat_tune.sh
# script.

echo sysstat_tune.sh
sysstat_tune.sh

print ""
print "tam_runstats.sh completed successfully"

