#!/bin/ksh

# example usage:
# 	cat request.log.2001-12-12-14-38-51 | xxx.sh

usage(){
	print Usage:  "cat <request.log file(s)> | $0 [all|noauth|auth]"
	print "        all: counts all pages (the default)"
	print "        noauth: counts only pages without 401 errors"
	print "        auth: counts only pages with 401 errors.  These are typically followed by an authentication"
	exit 1
}

if [ "X$1" = "X-h" ];then
	usage
fi

filter=all
if [ X$1 = X"auth" ];then
	filter=auth
elif [ X$1 = X"noauth" ];then
	filter=noauth
fi

# grep -v Unauth | \

awk -v filter=$filter 'BEGIN{

	interval = 20
}{
	d=substr($4,2,2)
	h=substr($4,14,2)
	m=substr($4,17,2)
	s=substr($4,20,2)
	if (d < pd){
		# print "DEBUG: d="d" pd="pd
		md+=pd
	}
	pd = d
	ts=(md+d)*24*60*60+h*60*60+m*60+s

	passed_filter = 0
	if (filter=="auth"){
		if ($(NF-1) == 401){
			passed_filter = 1
		}
	}else if (filter=="noauth"){
		if ($(NF-1) != 401){
			passed_filter = 1
		}
	}else{
			passed_filter = 1
	}

	if (passed_filter){
		count++
		total_size += $NF
	}

	if (bs=="") {
		bs=ts;
		pts=ts;
		# print "DEBUG: bs="bs
	}
	if (ts - pts >= interval){
		real_interval = ts - pts
		avg_page_size = 0
		if (count){
			avg_page_size = total_size/count/1000
		}
		printf("%d  %10.1f pages/sec  %10.3f KB avg page size  %10.3f KB/sec\n", \
			pts-bs, count/real_interval, avg_page_size, total_size/real_interval/1000)

		count = 0
		total_size = 0
		pts = ts
	}

}END{
		# print "DEBUG: ts="ts
		if (ts - pts >= 0){
			real_interval = ts - pts
			avg_page_size = 0
			if (count){
				avg_page_size = total_size/count/1000
			}
			printf("%d  %10.1f pages/sec  %10.3f KB avg page size  %10.3f KB/sec\n", \
				pts-bs, count/real_interval, avg_page_size, total_size/real_interval/1000)
		}
}'
